package abcm.cloning;

import java.util.Hashtable;
import java.util.Iterator;


/**
 * Convenience class for hashtables that are assumed
 * to have Strings as keys (and hence are NOT deep clones;
 * Strings are immutable) and objects implementing
 * Cloneable interface as objects.
 *
 * @param <K> key
 * @param <V> value
 *
 *  @author John T. Murphy <br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class ABCMDeepCloneableHashtable<K, V extends ABCMDeepCloneable> extends Hashtable<String, ABCMDeepCloneable> implements ABCMDeepCloneable{

  /** Serial ID  */
  private static final long serialVersionUID = 4761228613681830751L;

  /**
   * Gets a deep clone of this Hashtable, including clones of all
   * the objects it contains. Keys are not cloned.
   * @see abcm.cloning.ABCMDeepCloneable#getDeepClone(abcm.cloning.ABCMCloneStore)
   */
  public ABCMDeepCloneable getDeepClone(ABCMCloneStore cloneStore){
    ABCMDeepCloneableHashtable<K, V> clone = new ABCMDeepCloneableHashtable<K, V>();
    cloneStore.put(this, clone);
    Iterator i = keySet().iterator();
    while(i.hasNext()){
      String key = (String)i.next();
      Object toClone = get(key);
      if(toClone != null){
        ABCMDeepCloneable objectClone = cloneStore.getClone((ABCMDeepCloneable)toClone);
        if(objectClone == null){
          System.err.println("OBJECTCLONE IS NULL");
        }
        clone.put(key ,objectClone);
      }               
    }    
    return clone;
  }
  
  
}
