package abcm.db;

import java.io.File;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * Convenience class to wrap XML-to-HTML transformation.
 *  
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class ABCMXMLTransformer {
  
  /**
   * Get the HTML version of the xml data passed as a String,
   * using a specified xslt transform sheet.
   * @param xml String of xml to be transformed
   * @param xslt full pathname of transform sheet
   * @param includeHeaders whether HTML headers should be included in the result
   * @return a String that represents the xml data in HTML format.
   */
  public static String getHTML(String xml, String xslt, boolean includeHeaders){
    String retVal = "";
    
    try{
      TransformerFactory tf = TransformerFactory.newInstance();
      Transformer toHTML = tf.newTransformer(new StreamSource(xslt));
      if(!includeHeaders) toHTML.setParameter("OMIT_HTML_HEADERS","yes");
      StringWriter strOut = new StringWriter();
      toHTML.transform(
          new StreamSource(new StringReader(xml)),
          new StreamResult(strOut));
      retVal = strOut.toString();
    }catch(Exception e){  
      retVal = ErrorLogEntryMaker.HTMLEntry(e);
    }
    
    return retVal; 
  }

  /**
   * Get the HTML version of the xml data passed as a File,
   * using a specified xslt transform sheet.
   * @param xml String of xml to be transformed
   * @param xslt full pathname of transform sheet
   * @param includeHeaders whether HTML headers should be included in the result
   * @return a String that represents the xml data in HTML format.
   */
  public static String getHTML(File xml, String xslt, boolean includeHeaders){
    String retVal = "";
    
    try{
      TransformerFactory tf = TransformerFactory.newInstance();
      Transformer toHTML = tf.newTransformer(new StreamSource(xslt));
      if(!includeHeaders) toHTML.setParameter("OMIT_HTML_HEADERS","yes");
      StringWriter strOut = new StringWriter();
      toHTML.transform(
          new StreamSource(xml),
          new StreamResult(strOut));
      retVal = strOut.toString();
    }catch(Exception e){  
      retVal = ErrorLogEntryMaker.HTMLEntry(e);
    }
    
    return retVal; 
  }
  
  /**
   * Transform XML to an HTML format using a transformer
   * @param xml the XML to be transformed
   * @param xsltName the name of the stylesheet to be used in the transformation
   * @return the transformed result
   */
  public static String transform(String xml, String xsltName){
    String retVal = "";
    
    try{
      TransformerFactory tf = TransformerFactory.newInstance();
      Transformer toNewTransform = tf.newTransformer(new StreamSource(xsltName));
      StringWriter strOut = new StringWriter();
      toNewTransform.transform(
          new StreamSource(new StringReader(xml)),
          new StreamResult(strOut));
      retVal = strOut.toString();
    }catch(Exception e){  
      retVal = ErrorLogEntryMaker.HTMLEntry(e);
    }
    
    return retVal; 
  }
  
}
