package abcm.db;

/**
 * Only database manager objects will implement this interface,
 * and only some of these will. Specifically, only elements that
 * fall under the audit rules of the HWM Simulation will need
 * these methods.<br/><br/>
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public interface Auditable {

  /**
   * Indicates whether this object can be edited, given the audit
   * rules of the HWM Simulation and HWM Database.
   * 
   * Note that this does not refer to whether a specific user can
   * edit the object; generally only owners and administrators can
   * edit an object. Editable means that the owner or administrator
   * can or cannot modify the object given whether object has been
   * commented on or used in other contexts.
   * 
   * @param PK primary key of the object
   * @return true if the object can be edited; false if not
   */
  public boolean canBeEdited(long PK);
  
  /**
   * Returns the owner of this object.
   * @param PK Primary Key of the object
   * @return Primary Key of the Researcher who created or owns this object
   */
  public long getOwner(long PK);
  
  /**
   * Accepts a numeric argument that presumably represents the Primary Key
   * of an HWM researcher, and returns true if that researcher is the owner
   * of the object, false otherwise.
   * @param PK Primary Key of the object in question
   * @param userID Primary Key of the possible user
   * @return true if the user is, in fact, the owner of the object in question,
   * false if not
   */
  public boolean isOwner(long PK, long userID);
  
}
