package abcm.db;

import java.io.StringWriter;
import java.util.Date;

/**
 * This class does nothing except reformat an Exception object's information
 * into a prettier package.<br/><br/>
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class ErrorLogEntryMaker {

  /**
   * Gets the information from an Exception object and returns
   * a formatted version of it
   * @param e the Exception
   * @return the formatted version of the Exception's information
   */
  public static String entry(Exception e){
    StringWriter out = new StringWriter();
    out.write("\nERROR: " + (new Date(System.currentTimeMillis())).toString() + "\n");
    out.write("Message: " + e.getMessage() + "\n\n");
    out.write("Stack Trace: \n");
    StackTraceElement[] st = e.getStackTrace();
    for(int i = 0; i < st.length; i++){
      out.write(st[i].getClassName() + " in " + st[i].getFileName() + " method: " + st[i].getMethodName() + " line: " + st[i].getLineNumber() + "\n");    
    }
    return out.toString();
  }
  
  /**
   * Gets the information from an Exception object and returns
   * an HTML formatted version of it
   * @param e the Exception
   * @return the formatted version of the Exception's information
   */
  public static String HTMLEntry(Exception e){
    return (entry(e)).replaceAll("\n","<br>");
  }
    
}
