package abcm.db.components;

import java.util.Hashtable;

import abcm.db.syntaxengines.AbstractSQLSyntaxEngine;

/**
 * An ABCMdbDatum represents an item that can be saved to the database.
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class ABCMdbDatum {
  
  /** Table to which this Datum will be saved  */
  private ABCMdbTable table;
  /** Collection of fields and values to be saved  */
  private Hashtable   fieldsAndValues = new Hashtable();
  /**  Flag indicating whether this datum includes an identity PK. If so, then when the Datum is saved, the table will be instructed to SET IDENTITY INSERT OFF and then on again */
  private boolean     includesIdentityPK = false;
    
  /**
   * Constructor
   * @param t Table to which this Datum will be saved
   */
  public ABCMdbDatum(ABCMdbTable t){                    table = t;                        }
  
  /**
   * Adds a field and value to this Datum
   * @param field field to add
   * @param val value to add
   */
  public void addValue(ABCMdbField field, String val){  
    if(val != null) fieldsAndValues.put(field, val);  
  }
  
  /**
   * Adds a field and value to this Datum
   * @param field field to add
   * @param val value to add
   */
  public void addValue(ABCMdbField field, long val){
    fieldsAndValues.put(field, "" + val);
  }
  
  /**
   * Gets the SQL that will insert this Datum into the appropriate table
   * @param dbOwner name of the db owner (used in constructing the SQL)
   * @return the SQL to perform the insert
   */
  public String getInsertSQL(String dbOwner, AbstractSQLSyntaxEngine syntaxEngine){    return table.getInsertSQL(fieldsAndValues, dbOwner, syntaxEngine);   }  
  
  /**
   * Gets the table to which this Datum will be saved
   * @return the table to which this Datum will be saved
   */
  public ABCMdbTable getTable(){ return table; }
  
  /**
   * Gets the 'includes identity' flag
   * @return true if this Datum includes an identity field, false otherwise
   */
  public boolean getIncludesIdentityPK(){ return includesIdentityPK; }
  
  /**
   * Sets the 'includes identity' flag
   * @param includesIDPK new value for 'includes identity'
   */
  public void setIncludesIdentityPK(boolean includesIDPK){
    includesIdentityPK = includesIDPK;
  }
  
}
