package abcm.db.components;

import java.util.Vector;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.syntaxengines.AbstractSQLSyntaxEngine;

/**
 * Instances of ABCMdbView represent Views in the associated
 * SQL Server database 
 *
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public abstract class ABCMdbView extends ABCMdbAbstractViewable {
    
  /** Database Connection  */
  protected AbstractABCMDBGateway DB;
  
  /**
   * Constructor
   * @param viewName name of this view
   * @param db Instance of an ABCM Gateway to provide database access.
   */
  public ABCMdbView(String viewName, AbstractABCMDBGateway db){
    super(viewName);
    DB = db;
  }
  
  /** 
   * @see abcm.db.components.ABCMdbAbstractViewable#getCreateSQL(java.lang.String) 
   */
  public String[] getCreateSQL(String dbOwner, AbstractSQLSyntaxEngine syntaxEngine){
    String[] ret = new String[1];
    ret[0] = "";
    ret[0] =  "CREATE VIEW " + getName() + " AS \n";
    ret[0] += getSQL();
    return ret; 
  }
  
  /**
   * Implementing classes should be able to provide a single SQL statement
   * that returns all records
   * @return an SQL statement that returns all records of this view
   */
  public abstract String getSQL();
  
}
