
package abcm.db.components.views;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbView;
import abcm.db.components.tables.ABCMHistoriesTable;
import abcm.db.components.tables.ABCMResearchersTable;
import abcm.db.components.tables.ABCMTruthValuesTable;

public abstract class ABCMHistoriesAndResearchersView extends ABCMdbView {
  
  public ABCMHistoriesAndResearchersView(String viewName, AbstractABCMDBGateway db){ 
    super(viewName, db);
  }

  public abstract ABCMdbField getField_History_PK();
  public abstract ABCMdbField getField_History_Title();
  public abstract ABCMdbField getField_Researcher_PK();
  public abstract ABCMdbField getField_History_Description();
  public abstract ABCMdbField getField_Researcher_Name();
  public abstract ABCMdbField getField_CanBeEdited();
  public abstract ABCMdbField getField_TruthValue_Value();
  
  public String getSQL(){
    String dbOwner = DB.getOwner();
    
    ABCMHistoriesTable historiesTable = DB.getHistoriesTable();
        String histories = historiesTable.getFullName(dbOwner); 
        
        String HISTORY_PK          = historiesTable.getField_PK().getFullFieldName(dbOwner);
        String HISTORY_TITLE       = historiesTable.getField_Title().getFullFieldName(dbOwner);
        String RESEARCHER_FK       = historiesTable.getField_ResearcherFK().getFullFieldName(dbOwner);
        String HISTORY_DESC        = historiesTable.getField_Description().getFullFieldName(dbOwner);
        String HISTORY_TRUTH_VALUE = historiesTable.getField_TruthValueFK().getFullFieldName(dbOwner);
    
    ABCMResearchersTable researchersTable = DB.getResearchersTable();
        String researchers = researchersTable.getFullName(dbOwner);
        
        String RESEARCHER_PK   = researchersTable.getField_PK().getFullFieldName(dbOwner);
        String RESEARCHER_NAME = researchersTable.getField_ResearcherName().getFullFieldName(dbOwner);
        
    ABCMCanBeEdited_HistoriesView canBeEdited_HistoriesView = DB.getCanBeEdited_HistoriesView();
        String canBeEdited_Histories = canBeEdited_HistoriesView.getFullName(dbOwner);
        
        String CBE_HISTORY_PK = canBeEdited_HistoriesView.getField_History_PK().getFullFieldName(dbOwner);
        
    ABCMTruthValuesTable truthValuesTable = DB.getTruthValuesTable();
        String truthValues = truthValuesTable.getFullName(dbOwner);
        
        String TRUTH_VALUE_PK = truthValuesTable.getField_PK().getFullFieldName(dbOwner);
        String TRUTH_VALUE    = truthValuesTable.getField_TruthValue().getFullFieldName(dbOwner);
        
    return
       "SELECT " + 
          HISTORY_PK      + " AS " + getField_History_PK().getName()          + ", \n" + 
          HISTORY_TITLE   + " AS " + getField_History_Title().getName()       + ", \n" + 
          RESEARCHER_FK   + " AS " + getField_Researcher_PK().getName()       + ", \n" + 
          HISTORY_DESC    + " AS " + getField_History_Description().getName() + ", \n" + 
          RESEARCHER_NAME + " AS " + getField_Researcher_Name().getName()     + ", \n" + 
          CBE_HISTORY_PK  + " AS " + getField_CanBeEdited().getName()         + ", \n" + 
          TRUTH_VALUE     + " AS " + getField_TruthValue_Value().getName()    + "\n" + 
       "FROM  " + 
          histories + " INNER JOIN " +"\n" +           
          researchers + " ON " +"\n" + 
                RESEARCHER_FK + " = " + RESEARCHER_PK + " INNER JOIN " +"\n" + 
          truthValues + " ON " +"\n" + 
                HISTORY_TRUTH_VALUE + " = " + TRUTH_VALUE_PK + " LEFT OUTER JOIN " +"\n" + 
          canBeEdited_Histories + " ON " +"\n" + 
                HISTORY_PK + " = " + CBE_HISTORY_PK;
  }
  

}
