
package abcm.db.components.views;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbView;
import abcm.db.components.tables.ABCMHistoriesTable;
import abcm.db.components.tables.ABCMHistoryNarrativesTable;
import abcm.db.components.tables.ABCMNarrativeCommandsTable;
import abcm.db.components.tables.ABCMNarrativeInstructionsTable;
import abcm.db.components.tables.ABCMNarrativesTable;

public abstract class ABCMHistoriesNarrativesAndInstructionsView extends ABCMdbView {
  
  public ABCMHistoriesNarrativesAndInstructionsView(String viewName, AbstractABCMDBGateway db){ 
    super(viewName, db);
  }

  public abstract ABCMdbField getField_History_PK();
  public abstract ABCMdbField getField_CalculatedInstructionDate();
  public abstract ABCMdbField getField_NarrativeOrder();
  public abstract ABCMdbField getField_Narrative_PK();
  public abstract ABCMdbField getField_Narrative_Name();
  public abstract ABCMdbField getField_InstructionOrder();
  public abstract ABCMdbField getField_OriginalInstructionDate();
  public abstract ABCMdbField getField_Command_PK();
  public abstract ABCMdbField getField_Command();
  public abstract ABCMdbField getField_Arg1();
  public abstract ABCMdbField getField_Arg2();
  public abstract ABCMdbField getField_Arg3();
  public abstract ABCMdbField getField_Arg4();
  public abstract ABCMdbField getField_Arg5();
  public abstract ABCMdbField getField_Arg6();
  public abstract ABCMdbField getField_Arg7();
  public abstract ABCMdbField getField_Arg8();
  public abstract ABCMdbField getField_Arg9();
  public abstract ABCMdbField getField_Arg10();
  public abstract ABCMdbField getField_Comment();
  public abstract ABCMdbField getField_NumberOfArguments();
  public abstract ABCMdbField getField_IsSeasonal();

  public String getSQL(){
    String dbOwner = DB.getOwner();
    
    ABCMNarrativeInstructionsTable narrativeInstructionsTable = DB.getNarrativeInstructionsTable();
        String narrativeInstructions = narrativeInstructionsTable.getFullName(dbOwner);

        String NARR_INSTR_NARR_FK    = narrativeInstructionsTable.getField_NarrativeFK().getFullFieldName(dbOwner);
        String NARR_INSTR_COMMAND_FK = narrativeInstructionsTable.getField_NarrativeCommandFK().getFullFieldName(dbOwner);
        String NARR_INSTR_DATE       = narrativeInstructionsTable.getField_InstructionDate().getFullFieldName(dbOwner);
        
    ABCMNarrativesTable narrativesTable = DB.getNarrativesTable();
        String narratives = narrativesTable.getFullName(dbOwner);
        
        String NARR_PK          = narrativesTable.getField_PK().getFullFieldName(dbOwner);
        String NARR_BASE_DATE   = narrativesTable.getField_BaseDate().getFullFieldName(dbOwner);
        String NARR_IS_SEASONAL = narrativesTable.getField_Seasonal().getFullFieldName(dbOwner);

    ABCMHistoryNarrativesTable historyNarrativesTable = DB.getHistoryNarrativesTable();
        String historyNarratives = historyNarrativesTable.getFullName(dbOwner);

        String HISTORY_NARR_HISTORY_FK     = historyNarrativesTable.getField_HistoryFK().getFullFieldName(dbOwner);
        String HISTORY_NARR_NARR_FK        = historyNarrativesTable.getField_NarrativeFK().getFullFieldName(dbOwner);
        String HISTORY_NARR_STARTBASE_DATE = historyNarrativesTable.getField_BaseDate().getFullFieldName(dbOwner);
        
    ABCMHistoriesTable historiesTable = DB.getHistoriesTable();
        String histories = historiesTable.getFullName(dbOwner);

        String HISTORY_PK = historiesTable.getField_PK().getFullFieldName(dbOwner);
        
    ABCMNarrativeCommandsTable narrativeCommandsTable = DB.getNarrativeCommandsTable();
        String narrativeCommands = narrativeCommandsTable.getFullName(dbOwner);

        String NARR_COMMANDS_PK = narrativeCommandsTable.getField_PK().getFullFieldName(dbOwner);
    
    return
    "SELECT " +
         HISTORY_PK                                                                           + " AS " + getField_History_PK().getName() + ", \n" +
         "(" + NARR_INSTR_DATE                             + " - " + 
               NARR_BASE_DATE                              + " + " +   
               HISTORY_NARR_STARTBASE_DATE           + ")" + " - " + 
               NARR_IS_SEASONAL                            + " * " + 
              "(FLOOR((" + NARR_INSTR_DATE      + " - " + 
                           NARR_BASE_DATE       + " + " +
                           HISTORY_NARR_STARTBASE_DATE + ") / 365) * 365)"                     + " AS " + getField_CalculatedInstructionDate().getName() + ", \n" + 
          historyNarrativesTable.getField_Order().getFullFieldName(dbOwner)                    + " AS " + getField_NarrativeOrder().getName() + ", \n" +
          narrativeInstructionsTable.getField_NarrativeFK().getFullFieldName(dbOwner)          + " AS " + getField_Narrative_PK().getName() + ", \n" +
          narrativesTable.getField_Name().getFullFieldName(dbOwner)                            + " AS " + getField_Narrative_Name().getName() + ", \n" +
          narrativeInstructionsTable.getField_InstructionOrder().getFullFieldName(dbOwner)     + " AS " + getField_InstructionOrder().getName() + ", \n" +
          narrativeInstructionsTable.getField_InstructionDate().getFullFieldName(dbOwner)      + " AS " + getField_OriginalInstructionDate().getName() + ", \n" +
          narrativeInstructionsTable.getField_NarrativeCommandFK().getFullFieldName(dbOwner)   + " AS " + getField_Command_PK().getName() + ", \n" +
          narrativeCommandsTable.getField_Command().getFullFieldName(dbOwner)                  + " AS " + getField_Command().getName() + ", \n" +
          narrativeInstructionsTable.getField_Arg1().getFullFieldName(dbOwner)                 + " AS " + getField_Arg1().getName()  + ", \n" +
          narrativeInstructionsTable.getField_Arg2().getFullFieldName(dbOwner)                 + " AS " + getField_Arg2().getName()  + ", \n" +
          narrativeInstructionsTable.getField_Arg3().getFullFieldName(dbOwner)                 + " AS " + getField_Arg3().getName()  + ", \n" +
          narrativeInstructionsTable.getField_Arg4().getFullFieldName(dbOwner)                 + " AS " + getField_Arg4().getName()  + ", \n" +
          narrativeInstructionsTable.getField_Arg5().getFullFieldName(dbOwner)                 + " AS " + getField_Arg5().getName()  + ", \n" +
          narrativeInstructionsTable.getField_Arg6().getFullFieldName(dbOwner)                 + " AS " + getField_Arg6().getName()  + ", \n" +
          narrativeInstructionsTable.getField_Arg7().getFullFieldName(dbOwner)                 + " AS " + getField_Arg7().getName()  + ", \n" +
          narrativeInstructionsTable.getField_Arg8().getFullFieldName(dbOwner)                 + " AS " + getField_Arg8().getName()  + ", \n" +
          narrativeInstructionsTable.getField_Arg9().getFullFieldName(dbOwner)                 + " AS " + getField_Arg9().getName()  + ", \n" +
          narrativeInstructionsTable.getField_Arg10().getFullFieldName(dbOwner)                + " AS " + getField_Arg10().getName() + ", \n" +
          narrativeInstructionsTable.getField_Comment().getFullFieldName(dbOwner)              + " AS " + getField_Comment().getName() + ", \n" +
          narrativeCommandsTable.getField_NumberOfArgs().getFullFieldName(dbOwner)             + " AS " + getField_NumberOfArguments().getName() + ", \n" +
          narrativesTable.getField_Seasonal().getFullFieldName(dbOwner)                        + " AS " + getField_IsSeasonal().getName() + "\n" +
     "FROM " + histories + "\n" +
        "INNER JOIN " + historyNarratives + " ON\n" + 
            HISTORY_PK + " = " + HISTORY_NARR_HISTORY_FK + "\n" +
        "INNER JOIN " + narratives + " ON\n" + 
            HISTORY_NARR_NARR_FK + " = " + NARR_PK + "\n" +
        "INNER JOIN " + narrativeInstructions + " ON\n" + 
            NARR_PK + " = " + NARR_INSTR_NARR_FK + "\n" +
        "INNER JOIN " + narrativeCommands + " ON " + 
            NARR_INSTR_COMMAND_FK + " = " + NARR_COMMANDS_PK + "\n" +
     "ORDER BY (" + NARR_INSTR_DATE                             + " - " + 
               NARR_BASE_DATE                              + " + " +   
               HISTORY_NARR_STARTBASE_DATE           + ")" + " - " + 
               NARR_IS_SEASONAL                            + " * " + 
              "(FLOOR((" + NARR_INSTR_DATE      + " - " + 
                           NARR_BASE_DATE       + " + " +
                           HISTORY_NARR_STARTBASE_DATE + ") / 365) * 365)" + ",\n" + 
                    historyNarrativesTable.getField_Order().getFullFieldName(dbOwner) + ", " +  
                    narrativeInstructionsTable.getField_InstructionOrder().getFullFieldName(dbOwner);
  }
  
  
}
