
package abcm.db.components.views;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbView;
import abcm.db.components.tables.ABCMProbeSetProbesTable;
import abcm.db.components.tables.ABCMProbeSetsTable;
import abcm.db.components.tables.ABCMProbesTable;
import abcm.db.components.tables.ABCMResearchersTable;

public abstract class ABCMProbeSetProbesView extends ABCMdbView {
  
  public ABCMProbeSetProbesView(String viewName, AbstractABCMDBGateway db){ 
    super(viewName, db);
  }

  public abstract ABCMdbField getField_Researcher_Name();
  public abstract ABCMdbField getField_ProbeSet_PK();
  public abstract ABCMdbField getField_ProbeSet_Name();
  public abstract ABCMdbField getField_ProbeSet_DateCreated();
  public abstract ABCMdbField getField_Researcher_PK();
  public abstract ABCMdbField getField_ProbeSet_Desc();
  public abstract ABCMdbField getField_Probe_Name();
  public abstract ABCMdbField getField_Probe_Description();
  public abstract ABCMdbField getField_Probe_Date_Created();
  public abstract ABCMdbField getField_Probe_Researcher_FK();
  public abstract ABCMdbField getField_DataSeries_Name();
  public abstract ABCMdbField getField_DataSeries_Domain();
  public abstract ABCMdbField getField_Probe_IsNumeric();
  public abstract ABCMdbField getField_Probe_IsMonthly();
  public abstract ABCMdbField getField_Probe_BaseDate();
  public abstract ABCMdbField getField_Probe_Frequency();
  public abstract ABCMdbField getField_Probe_PK();
  public abstract ABCMdbField getField_Probe_Retrieve();
  
  public String getSQL(){
    String dbOwner = DB.getOwner();

    ABCMResearchersTable researchersTable = DB.getResearchersTable();
        String researchers = researchersTable.getFullName(dbOwner);
        
        String RESEARCHER_PK   = researchersTable.getField_PK().getFullFieldName(dbOwner);
        String RESEARCHER_NAME = researchersTable.getField_ResearcherName().getFullFieldName(dbOwner);

    ABCMProbeSetsTable probeSetsTable = DB.getProbeSetsTable();
        String probeSets = probeSetsTable.getFullName(dbOwner);

        String PROBE_SET_PK            = probeSetsTable.getField_PK().getFullFieldName(dbOwner);
        String PROBE_SET_NAME          = probeSetsTable.getField_Name().getFullFieldName(dbOwner);
        String PROBE_SET_DATE          = probeSetsTable.getField_DateCreated().getFullFieldName(dbOwner);
        String PROBE_SET_RESEARCHER_FK = probeSetsTable.getField_ResearcherFK().getFullFieldName(dbOwner);
        String PROBE_SET_DESC          = probeSetsTable.getField_Description().getFullFieldName(dbOwner);

    ABCMProbesTable probesTable = DB.getProbesTable();
        String probes = probesTable.getFullName(dbOwner);
        
        String PROBE_NAME               = probesTable.getField_Name().getFullFieldName(dbOwner);
        String PROBE_DESC               = probesTable.getField_Description().getFullFieldName(dbOwner);
        String PROBE_DATE               = probesTable.getField_DateCreated().getFullFieldName(dbOwner);
        String PROBE_RESEARCHER_FK      = probesTable.getField_ResearcherFK().getFullFieldName(dbOwner);
        String PROBE_DATA_SERIES_NAME   = probesTable.getField_DataSeriesName().getFullFieldName(dbOwner);
        String PROBE_DATA_SERIES_DOMAIN = probesTable.getField_DataSeriesDomain().getFullFieldName(dbOwner);
        String PROBE_IS_NUMERIC         = probesTable.getField_IsNumeric().getFullFieldName(dbOwner);
        String PROBE_IS_MONTHLY         = probesTable.getField_IsMonthly().getFullFieldName(dbOwner);
        String PROBE_BASE_DATE          = probesTable.getField_BaseDate().getFullFieldName(dbOwner);
        String PROBE_FREQ               = probesTable.getField_Frequency().getFullFieldName(dbOwner);
        String PROBE_PK                 = probesTable.getField_PK().getFullFieldName(dbOwner);
        String PROBE_RETRIEVE           = probesTable.getField_Retrieve().getFullFieldName(dbOwner);

    ABCMProbeSetProbesTable probeSetProbesTable = DB.getProbeSetProbesTable();
        String probeSetProbes = probeSetProbesTable.getFullName(dbOwner);
    
        String PROBE_SET_PROBES_PROBE_SET_FK = probeSetProbesTable.getField_ProbeSetFK().getFullFieldName(dbOwner);
        String PROBE_SET_PROBES_PROBE_FK     = probeSetProbesTable.getField_ProbeFK().getFullFieldName(dbOwner);
    return
     "SELECT " +
         RESEARCHER_NAME           + " AS " + getField_Researcher_Name().getName()      + ", \n" +
         PROBE_SET_PK              + " AS " + getField_ProbeSet_PK().getName()          + ", \n" +
         PROBE_SET_NAME            + " AS " + getField_ProbeSet_Name().getName()        + ", \n" +
         PROBE_SET_DATE            + " AS " + getField_ProbeSet_DateCreated().getName() + ", \n" +
         PROBE_SET_RESEARCHER_FK   + " AS " + getField_Researcher_PK().getName()        + ", \n" +
         PROBE_SET_DESC            + " AS " + getField_ProbeSet_Desc().getName()        + ", \n" +
         PROBE_NAME                + " AS " + getField_Probe_Name().getName()           + ", \n" +
         PROBE_DESC                + " AS " + getField_Probe_Description().getName()    + ", \n" +
         PROBE_DATE                + " AS " + getField_Probe_Date_Created().getName()   + ", \n" +
         PROBE_RESEARCHER_FK       + " AS " + getField_Probe_Researcher_FK().getName()  + ", \n" +
         PROBE_DATA_SERIES_NAME    + " AS " + getField_DataSeries_Name().getName()      + ", \n" +
         PROBE_DATA_SERIES_DOMAIN  + " AS " + getField_DataSeries_Domain().getName()    + ", \n" +
         PROBE_IS_NUMERIC          + " AS " + getField_Probe_IsNumeric().getName()      + ", \n" +
         PROBE_IS_MONTHLY          + " AS " + getField_Probe_IsMonthly().getName()      + ", \n" +
         PROBE_BASE_DATE           + " AS " + getField_Probe_BaseDate().getName()       + ", \n" +
         PROBE_PK                  + " AS " + getField_Probe_PK().getName()             + ", \n" +
         PROBE_RETRIEVE            + " AS " + getField_Probe_Retrieve().getName()       + ", \n" +
         PROBE_FREQ                + " AS " + getField_Probe_Frequency().getName()      + "\n" +
      "FROM   " + probeSets + " " +
          "INNER JOIN " + probeSetProbes + " ON " +"\n" + 
              PROBE_SET_PK + " = " + PROBE_SET_PROBES_PROBE_SET_FK + " " +
          "INNER JOIN " + probes + " ON " +"\n" + 
              PROBE_SET_PROBES_PROBE_FK + " = " + PROBE_PK + " " +
          "INNER JOIN " + researchers + " ON " +"\n" + 
              PROBE_SET_RESEARCHER_FK + " = " + RESEARCHER_PK;
  }
  

}
