
package abcm.db.components.views;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbView;
import abcm.db.components.tables.ABCMParameterSetsTable;
import abcm.db.components.tables.ABCMProbeSetsTable;
import abcm.db.components.tables.ABCMResearchersTable;
import abcm.db.components.tables.ABCMRunModesTable;
import abcm.db.components.tables.ABCMSimulationRunsTable;
import abcm.db.components.tables.ABCMSimulationsTable;
import abcm.db.components.tables.ABCMTruthValuesTable;

public abstract class ABCMSimulationRunsAndResearchersView extends ABCMdbView {
  
  public ABCMSimulationRunsAndResearchersView(String viewName, AbstractABCMDBGateway db){ 
    super(viewName, db);
  }

  public abstract ABCMdbField getField_SimulationRun_PK();
  public abstract ABCMdbField getField_SimulationRun_DateTimeBegun();
  public abstract ABCMdbField getField_Simulation_PK();
  public abstract ABCMdbField getField_Simulation_Title();
  public abstract ABCMdbField getField_ParameterSet_PK();
  public abstract ABCMdbField getField_ParameterSet_Name();
  public abstract ABCMdbField getField_ProbeSet_PK();
  public abstract ABCMdbField getField_ProbeSet_Name();
  public abstract ABCMdbField getField_SimulationRunMode_PK();
  public abstract ABCMdbField getField_Researcher_PK();
  public abstract ABCMdbField getField_SimulationRun_Notes();
  public abstract ABCMdbField getField_SimulationRun_RandomNumberSeed();
  public abstract ABCMdbField getField_SimulationRun_ReRun_FK();
  public abstract ABCMdbField getField_Researcher_Name();
  public abstract ABCMdbField getField_SimulationRunMode_Type();
  public abstract ABCMdbField getField_CombinedTruthValue();
  public abstract ABCMdbField getField_TruthValue();

  public String getSQL(){
    String dbOwner = DB.getOwner();
    
    ABCMSimulationRunsTable              simulationRunsTable       = DB.getSimulationRunsTable();
    ABCMSimulationsTable                 simulationsTable          = DB.getSimulationsTable();
    ABCMParameterSetsTable               paramSetsTable            = DB.getParameterSetsTable();
    ABCMProbeSetsTable                   probeSetsTable            = DB.getProbeSetsTable();
    ABCMRunModesTable                    runModesTable             = DB.getRunModesTable();
    ABCMResearchersTable                 researchersTable          = DB.getResearchersTable();
    ABCMSimulationsWithTruthValuesView   simsWithTruthValues       = DB.getSimulationsWithTruthValuesView();
    ABCMParameterSetsAndTruthValuesView  paramSetsWithTruthValues  = DB.getParameterSetsAndTruthValuesView();
    
    ABCMSimulationRunsAndResearchersSTEP1View simulationRunsAndResearchersSTEP1View = DB.getSimulationRunsAndResearchersSTEP1View();
        String simulationRunsAndResearchersSTEP1 = simulationRunsAndResearchersSTEP1View.getFullName(dbOwner);

    ABCMTruthValuesTable truthValuesTable = DB.getTruthValuesTable();
        String truthValues = truthValuesTable.getFullName(dbOwner);

    
    return
    "SELECT     " +
        simulationRunsAndResearchersSTEP1View.getField_SimulationRun_PK().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_PK().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_SimulationRun_DateTimeBegun().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_DateTimeBegun().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_Simulation_PK().getFullFieldName(dbOwner) + " AS " + getField_Simulation_PK().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_Simulation_Title().getFullFieldName(dbOwner) + " AS " + getField_Simulation_Title().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_ParameterSet_PK().getFullFieldName(dbOwner) + " AS " + getField_ParameterSet_PK().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_ParameterSet_Name().getFullFieldName(dbOwner) + " AS " + getField_ParameterSet_Name().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_ProbeSet_PK().getFullFieldName(dbOwner) + " AS " + getField_ProbeSet_PK().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_ProbeSet_Name().getFullFieldName(dbOwner) + " AS " + getField_ProbeSet_Name().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_SimulationRunMode_PK().getFullFieldName(dbOwner) + " AS " + getField_SimulationRunMode_PK().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_Researcher_PK().getFullFieldName(dbOwner) + " AS " + getField_Researcher_PK().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_SimulationRun_Notes().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_Notes().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_SimulationRun_RandomNumberSeed().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_RandomNumberSeed().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_SimulationRun_ReRun_FK().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_ReRun_FK().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_Researcher_Name().getFullFieldName(dbOwner) + " AS " + getField_Researcher_Name().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_SimulationRunMode_Type().getFullFieldName(dbOwner) + " AS " + getField_SimulationRunMode_Type().getName() + ", \n" +
        simulationRunsAndResearchersSTEP1View.getField_CombinedTruthValue().getFullFieldName(dbOwner) + " AS " + getField_CombinedTruthValue().getName() + ", \n" +
        truthValuesTable.getField_TruthValue().getFullFieldName(dbOwner) + " AS " + getField_TruthValue().getName() + "\n" +
    "FROM " + simulationRunsAndResearchersSTEP1View.getName() + " \n" +
        "JOIN " + truthValuesTable.getFullName(dbOwner) + " ON\n" + 
              simulationRunsAndResearchersSTEP1View.getField_CombinedTruthValue().getFullFieldName(dbOwner) + " = " + truthValuesTable.getField_PK().getFullFieldName(dbOwner);
//    "SELECT " + 
//    getField_SimulationRun_PK().getName() + " AS " + getField_SimulationRun_PK().getName() + ", \n" +
//    getField_SimulationRun_PK().getName() + " AS " + getField_SimulationRun_PK().getName() + ", \n" +
//    getField_Simulation_PK().getName() + " AS " + getField_Simulation_PK().getName() + ", \n" +
//    getField_Simulation_Title().getName() + " AS " + getField_Simulation_Title().getName() + ", \n" +
//    getField_ParameterSet_PK().getName() + " AS " + getField_ParameterSet_PK().getName() + ", \n" +
//    getField_ParameterSet_Name().getName() + " AS " + getField_ParameterSet_Name().getName() + ", \n" +
//    getField_ProbeSet_PK().getName() + " AS " + getField_ProbeSet_PK().getName() + ", \n" +
//    getField_ProbeSet_Name().getName() + " AS " + getField_ProbeSet_Name().getName() + ", \n" +
//    getField_SimulationRunMode_PK().getName() + " AS " + getField_SimulationRunMode_PK().getName() + ", \n" +
//    getField_Researcher_PK().getName() + " AS " + getField_Researcher_PK().getName() + ", \n" +
//    getField_SimulationRun_Notes().getName() + " AS " + getField_SimulationRun_Notes().getName() + ", \n" +
//    getField_SimulationRun_RandomNumberSeed().getName() + " AS " + getField_SimulationRun_RandomNumberSeed().getName() + ", \n" +
//    getField_SimulationRun_ReRun_FK().getName() + " AS " + getField_SimulationRun_ReRun_FK().getName() + ", \n" +
//    getField_Researcher_Name().getName() + " AS " + getField_Researcher_Name().getName() + ", \n" +
//    getField_SimulationRunMode_Type().getName() + " AS " + getField_SimulationRunMode_Type().getName() + ", \n" +
//    "MAX(SIMULATIONTRUTHVALUE, PARAMSETTRUTHVALUE)"  + " AS " + getField_CombinedTruthValue().getName() + " \n" +
//   "FROM (" +  
//    "SELECT " +
//      simulationRunsTable.getField_PK().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_PK().getName() + ", \n" +
//      simulationRunsTable.getField_DateCreated().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_DateTimeBegun().getName() + ", \n" +
//      simulationRunsTable.getField_SimulationFK().getFullFieldName(dbOwner) + " AS " + getField_Simulation_PK().getName() + ", \n" +
//      simulationsTable.getField_Title().getFullFieldName(dbOwner) + " AS " + getField_Simulation_Title().getName() + ", \n" +
//      simulationRunsTable.getField_ParameterSetFK().getFullFieldName(dbOwner) + " AS " + getField_ParameterSet_PK().getName() + ", \n" +
//      paramSetsTable.getField_Name().getFullFieldName(dbOwner) + " AS " + getField_ParameterSet_Name().getName() + ", \n" +
//      simulationRunsTable.getField_ProbeSetFK().getFullFieldName(dbOwner) + " AS " + getField_ProbeSet_PK().getName() + ", \n" +
//      probeSetsTable.getField_Name().getFullFieldName(dbOwner) + " AS " + getField_ProbeSet_Name().getName() + ", \n" +
//      simulationRunsTable.getField_RunModeFK().getFullFieldName(dbOwner) + " AS " + getField_SimulationRunMode_PK().getName() + ", \n" +
//      simulationRunsTable.getField_ResearcherFK().getFullFieldName(dbOwner) + " AS " + getField_Researcher_PK().getName() + ", \n" +
//      simulationRunsTable.getField_Notes().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_Notes().getName() + ", \n" +
//      simulationRunsTable.getField_RandomNumberSeed().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_RandomNumberSeed().getName() + ", \n" +
//      simulationRunsTable.getField_RerunID().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_ReRun_FK().getName() + ", \n" +
//      researchersTable.getField_ResearcherName().getFullFieldName(dbOwner) + " AS " + getField_Researcher_Name().getName() + ", \n" +
//      runModesTable.getField_RunMode().getFullFieldName(dbOwner) + " AS " + getField_SimulationRunMode_Type().getName() + ", \n" +
//      simsWithTruthValues.getField_TruthValue().getFullFieldName(dbOwner) + " AS " + "SIMULATIONTRUTHVALUE" + ", \n" +
//      paramSetsWithTruthValues.getField_TruthValue().getFullFieldName(dbOwner) + " AS " + " PARAMSETTRUTHVALUE " + "\n" +
//      "FROM " + simulationRunsTable.getFullName(dbOwner) + "\n" +
//        "JOIN " + simulationsTable.getFullName(dbOwner) + " " + 
//            "ON (" + simulationRunsTable.getField_SimulationFK().getFullFieldName(dbOwner) + " = " + simulationsTable.getField_PK().getFullFieldName(dbOwner) + ")\n" +
//        "JOIN " + paramSetsTable.getFullName(dbOwner) + " " +
//            "ON (" + simulationRunsTable.getField_ParameterSetFK().getFullFieldName(dbOwner) + " = " + paramSetsTable.getField_PK().getFullFieldName(dbOwner) + ")\n" +
//        "JOIN " + probeSetsTable.getFullName(dbOwner) + " " +
//            "ON (" + simulationRunsTable.getField_ProbeSetFK().getFullFieldName(dbOwner) + " = " + probeSetsTable.getField_PK().getFullFieldName(dbOwner) + ")\n" +
//        "JOIN " + researchersTable.getFullName(dbOwner) + " " +
//            "ON (" + simulationRunsTable.getField_ResearcherFK().getFullFieldName(dbOwner) + " = " + researchersTable.getField_PK().getFullFieldName(dbOwner) + ")\n" +
//        "JOIN " + runModesTable.getFullName(dbOwner) + " " +
//            "ON (" + simulationRunsTable.getField_RunModeFK().getFullFieldName(dbOwner) + " = " + runModesTable.getField_PK().getFullFieldName(dbOwner) + ")\n" +
//        "JOIN " + simsWithTruthValues.getFullName(dbOwner) + " " +
//            "ON (" + simulationRunsTable.getField_SimulationFK().getFullFieldName(dbOwner) + " = " + simsWithTruthValues.getField_Simulation_PK().getFullFieldName(dbOwner) + ")\n" +
//        "JOIN " + paramSetsWithTruthValues.getFullName(dbOwner) + " " +
//            "ON (" + simulationRunsTable.getField_ParameterSetFK().getFullFieldName(dbOwner) + " = " + paramSetsWithTruthValues.getField_ParameterSet_PK().getFullFieldName(dbOwner) + ")\n" +                                     
//      ");";
//  truthValuesTable.getField_TruthValue().getFullFieldName(dbOwner) + " AS " + getField_TruthValue().getName() + "\n" +
    
    
  }
  
  
}
