
package abcm.db.components.views;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbView;
import abcm.db.components.tables.ABCMAnalysesTable;
import abcm.db.components.tables.ABCMResearchersTable;
import abcm.db.components.tables.ABCMSummariesTable;
import abcm.db.components.tables.ABCMSummaryAnalysesTable;

public abstract class ABCMSummaryAnalysesAndDetailsView extends ABCMdbView {
  
  public ABCMSummaryAnalysesAndDetailsView(String viewName, AbstractABCMDBGateway db){ 
    super(viewName, db);
  }

  public abstract ABCMdbField getField_SummaryAnalysis_PK();
  public abstract ABCMdbField getField_Summary_PK();
  public abstract ABCMdbField getField_Summary_Title();
  public abstract ABCMdbField getField_Analysis_PK();
  public abstract ABCMdbField getField_AnalysisOrder();
  public abstract ABCMdbField getField_Analysis_Title();
  public abstract ABCMdbField getField_Analysis_DateCreated();
  public abstract ABCMdbField getField_Analysis_Desc();
  public abstract ABCMdbField getField_Researcher_LastNameFirstName();
  

  public String getSQL(){
    String dbOwner = DB.getOwner();

    ABCMSummaryAnalysesTable summaryAnalysesTable = DB.getSummaryAnalysesTable();
        String summaryAnalyses = summaryAnalysesTable.getFullName(dbOwner);

    ABCMAnalysesTable analysesTable = DB.getAnalysesTable();
        String analyses = analysesTable.getFullName(dbOwner);
    
    ABCMResearchersTable researchersTable = DB.getResearchersTable();
        String researchers = researchersTable.getFullName(dbOwner);

    ABCMSummariesTable summariesTable = DB.getSummariesTable();
        String summaries = summariesTable.getFullName(dbOwner);

    
    return
   "SELECT " +
     summaryAnalysesTable.getField_PK().getFullFieldName(dbOwner)            + " AS " + getField_SummaryAnalysis_PK().getName() + ", \n" +
     summaryAnalysesTable.getField_SummaryFK().getFullFieldName(dbOwner)     + " AS " + getField_Summary_PK().getName() + ", \n" +
     analysesTable.getField_Title().getFullFieldName(dbOwner)                + " AS " + getField_Analysis_Title().getName() + ", \n" +
     analysesTable.getField_DateCreated().getFullFieldName(dbOwner)          + " AS " + getField_Analysis_DateCreated().getName() + ", \n" +
     analysesTable.getField_Description().getFullFieldName(dbOwner)          + " AS " + getField_Analysis_Desc().getName() + ", \n" +
     researchersTable.getField_LastNameFirstName().getFullFieldName(dbOwner) + " AS " + getField_Researcher_LastNameFirstName().getName() + ", \n" +
     summariesTable.getField_Title().getFullFieldName(dbOwner)               + " AS " + getField_Summary_Title().getName() + ", \n" +
     summaryAnalysesTable.getField_AnalysisFK().getFullFieldName(dbOwner)    + " AS " + getField_Analysis_PK().getName() + "\n" +     
    "FROM " + summaryAnalyses + " " +
        "INNER JOIN " + analyses + " ON\n" + 
           summaryAnalysesTable.getField_AnalysisFK().getFullFieldName(dbOwner) + " = " + analysesTable.getField_PK().getFullFieldName(dbOwner) + "\n" + 
        "INNER JOIN " + researchers + " ON\n" +
           analysesTable.getField_ResearcherFK().getFullFieldName(dbOwner) + " = " + researchersTable.getField_PK().getFullFieldName(dbOwner) + "\n" + 
        "INNER JOIN " + summaries + " ON\n" +
           summaryAnalysesTable.getField_SummaryFK().getFullFieldName(dbOwner) + " = " + summariesTable.getField_PK().getFullFieldName(dbOwner);
    
    /* Revisions */
    /* SELECT     jtmurphy.tblSummaryAnalyses.intSummaryID, jtmurphy.tblAnalyses.vcharAnalysisTitle, jtmurphy.tblAnalyses.dteDateCreated, 
                      jtmurphy.tblAnalyses.txtDescription, jtmurphy.tblResearchers.vcharResearcherLastFirst, jtmurphy.tblSummaries.vcharSummaryTitle, 
                      jtmurphy.tblSummaryAnalyses.intAnalysisID
       FROM         jtmurphy.tblSummaryAnalyses INNER JOIN
                      jtmurphy.tblAnalyses ON jtmurphy.tblSummaryAnalyses.intAnalysisID = jtmurphy.tblAnalyses.intAnalysisID INNER JOIN
                      jtmurphy.tblResearchers ON jtmurphy.tblAnalyses.intResearcherID = jtmurphy.tblResearchers.intResearcherID INNER JOIN
                      jtmurphy.tblSummaries ON jtmurphy.tblSummaryAnalyses.intSummaryID = jtmurphy.tblSummaries.intSummaryID */
    
    
//    /* SELECT     jtmurphy.tblSummaryAnalyses.intSummaryID, jtmurphy.tblAnalyses.vcharAnalysisTitle, jtmurphy.tblAnalyses.dteDateCreated, 
//                      jtmurphy.tblAnalyses.txtDescription, jtmurphy.tblResearchers.vcharResearcherLastFirst, jtmurphy.tblSummaries.vcharSummaryTitle
//       FROM         jtmurphy.tblSummaryAnalyses INNER JOIN
//                      jtmurphy.tblAnalyses ON jtmurphy.tblSummaryAnalyses.intAnalysisID = jtmurphy.tblAnalyses.intAnalysisID INNER JOIN
//                      jtmurphy.tblResearchers ON jtmurphy.tblAnalyses.intResearcherID = jtmurphy.tblResearchers.intResearcherID INNER JOIN
//                      jtmurphy.tblSummaries ON jtmurphy.tblSummaryAnalyses.intSummaryID = jtmurphy.tblSummaries.intSummaryID
//                      
//                      */
    
//    setSQL(   "SELECT " +summaryAnalysesTable.getField_SummaryFK().getFullFieldName(dbOwner) + " AS intSummaryID, " +"\n" + 
//                   analysesTable.getField_Title().getFullFieldName(dbOwner) + " AS vcharAnalysisTitle, " +"\n" + 
//                   analysesTable.getField_DateCreated().getFullFieldName(dbOwner) + " AS dteDateCreated, " +"\n" + 
//                   analysesTable.getField_Description().getFullFieldName(dbOwner) + " AS txtDescription, " +"\n" + 
//                   researchersTable.getField_LastNameFirstName().getFullFieldName(dbOwner) + " AS vcharResearcherLastFirst " +"\n" + 
//           "FROM   " + summaryAnalysesTable.getFullName(dbOwner) + " INNER JOIN " +"\n" + 
//                        analysesTable.getFullName(dbOwner) + " ON " +"\n" + 
//                        summaryAnalysesTable.getField_AnalysisFK().getFullFieldName(dbOwner) + " = " + analysesTable.getField_PK().getFullFieldName(dbOwner) + " INNER JOIN " +"\n" + 
//                        researchersTable.getFullName(dbOwner) + " ON " +"\n" + 
//                        analysesTable.getField_ResearcherFK().getFullFieldName(dbOwner) + " = jtmurphy.tblResearchers.intResearcherID ");

  }
  
}
