
package abcm.db.components.views;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbView;
import abcm.db.components.tables.ABCMDataModelsTable;
import abcm.db.components.tables.ABCMHistoriesTable;
import abcm.db.components.tables.ABCMResearchersTable;
import abcm.db.components.tables.ABCMSimulationConfigurationsTable;

public abstract class ABCMSummaryDataModelsAndDetailsView extends ABCMdbView {
  
  public ABCMSummaryDataModelsAndDetailsView(String viewName, AbstractABCMDBGateway db){ 
    super(viewName, db);
  }

  public abstract ABCMdbField getField_Summary_PK();
  public abstract ABCMdbField getField_DataModel_PK();
  public abstract ABCMdbField getField_DataModel_Name();
  public abstract ABCMdbField getField_DataModel_Description();
  public abstract ABCMdbField getField_DataModel_DateCreated();
  public abstract ABCMdbField getField_Researcher_Name();
  public abstract ABCMdbField getField_SimulationConfiguration_Name();
  public abstract ABCMdbField getField_History_Title();

  public String getSQL(){
    String dbOwner = DB.getOwner();

    ABCMSummaryDataModelsView summaryDataModelsView = DB.getSummaryDataModelsView();
        String summaryDataModels = summaryDataModelsView.getFullName(dbOwner);

    ABCMDataModelsTable dataModelsTable = DB.getDataModelsTable();
        String dataModels = dataModelsTable.getFullName(dbOwner);

    ABCMResearchersTable researchersTable = DB.getResearchersTable();
        String researchers = researchersTable.getFullName(dbOwner);

    ABCMSimulationConfigurationsTable simulationConfigurationsTable = DB.getSimulationConfigurationsTable();
        String simulationConfigurations = simulationConfigurationsTable.getFullName(dbOwner);

    ABCMHistoriesTable historiesTable = DB.getHistoriesTable();
        String histories = historiesTable.getFullName(dbOwner);

    
    return
    "SELECT " +
        summaryDataModelsView.getField_Summary_PK().getFullFieldName(dbOwner) + " AS intSummaryID, " +"\n" + 
        dataModelsTable.getField_PK().getFullFieldName(dbOwner) + " AS intDataModelID, " +"\n" + 
        dataModelsTable.getField_Name().getFullFieldName(dbOwner) + " AS varcharDataModelName, " +"\n" + 
        dataModelsTable.getField_Description().getFullFieldName(dbOwner) + " AS txtDescription, " +"\n" + 
        dataModelsTable.getField_DateCreated().getFullFieldName(dbOwner) + " AS dteDateCreated, " +"\n" + 
        researchersTable.getField_ResearcherName().getFullFieldName(dbOwner) + " AS vcharResearcherName, " +"\n" + 
        simulationConfigurationsTable.getField_Name().getFullFieldName(dbOwner) + " AS vcharSimConfigName, " +"\n" + 
        historiesTable.getField_Title().getFullFieldName(dbOwner) + " AS vcharHistoryTitle " +"\n" + 
    "FROM "  + summaryDataModels + "\n" +
        "INNER JOIN " + dataModels + " ON\n" + 
              summaryDataModelsView.getField_DataModel_PK().getFullFieldName(dbOwner) + " = " + dataModelsTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
        "INNER JOIN " + histories + " ON\n" + 
               dataModelsTable.getField_HistoryFK().getFullFieldName(dbOwner) + " = " + historiesTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
        "INNER JOIN " + simulationConfigurations + " ON\n" + 
               dataModelsTable.getField_ConfigurationFK().getFullFieldName(dbOwner) + " = " + simulationConfigurationsTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
        "INNER JOIN " + researchers + " ON\n" + 
               dataModelsTable.getField_ResearcherFK().getFullFieldName(dbOwner) + " = " + researchersTable.getField_PK().getFullFieldName(dbOwner);
  }
  
  
}
