
package abcm.db.components.views;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbView;
import abcm.db.components.tables.ABCMParameterSetsTable;
import abcm.db.components.tables.ABCMProbeSetsTable;
import abcm.db.components.tables.ABCMResearchersTable;
import abcm.db.components.tables.ABCMSimulationRunsTable;
import abcm.db.components.tables.ABCMSimulationsTable;

public abstract class ABCMSummaryRunsAndDetailsView extends ABCMdbView {
  
  public ABCMSummaryRunsAndDetailsView(String viewName, AbstractABCMDBGateway db){ 
    super(viewName, db);
  }

  public abstract ABCMdbField getField_Summary_PK();
  public abstract ABCMdbField getField_SimulationRun_PK();
  public abstract ABCMdbField getField_Researcher_Name();
  public abstract ABCMdbField getField_SimulationRun_DateTimeBegun();
  public abstract ABCMdbField getField_SimulationRun_Notes();
  public abstract ABCMdbField getField_SimulationRun_RandomNumberSeed();
  public abstract ABCMdbField getField_SimulationRun_ReRun_FK();
  public abstract ABCMdbField getField_Simulation_Title();
  public abstract ABCMdbField getField_ParameterSet_Name();
  public abstract ABCMdbField getField_ProbeSet_Name();
  public abstract ABCMdbField getField_TruthValue();
  
  public String getSQL(){
    String dbOwner = DB.getOwner();
    
    ABCMSummaryRunsView summaryRunsView = DB.getSummaryRunsView();
    ABCMSimulationRunsTable simulationRunsTable = DB.getSimulationRunsTable();
    ABCMSimulationsTable simulationsTable = DB.getSimulationsTable();
    ABCMParameterSetsTable parameterSetsTable = DB.getParameterSetsTable();
    ABCMProbeSetsTable probeSetsTable = DB.getProbeSetsTable();
    ABCMResearchersTable researchersTable = DB.getResearchersTable();
    ABCMSimulationRunsAndResearchersView simulationRunsAndResearchersView = DB.getSimulationRunsAndResearchersView();

    String summaryRuns = summaryRunsView.getFullName(dbOwner);

    return
     "SELECT " +
         summaryRunsView.getField_Summary_PK().getFullFieldName(dbOwner) + " AS " + getField_Summary_PK().getName() + ", \n" + 
         simulationRunsTable.getField_PK().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_PK().getName() + ", \n" +
         researchersTable.getField_ResearcherName().getFullFieldName(dbOwner) + " AS " + getField_Researcher_Name().getName() + ", \n" +
         simulationRunsTable.getField_DateCreated().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_DateTimeBegun().getName() + ", \n" +
         simulationRunsTable.getField_Notes().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_Notes().getName() + ", \n" +
         simulationRunsTable.getField_RandomNumberSeed().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_RandomNumberSeed().getName() + ", \n" +
         simulationRunsTable.getField_RerunID().getFullFieldName(dbOwner) + " AS " + getField_SimulationRun_ReRun_FK().getName() + ", \n" +
         simulationsTable.getField_Title().getFullFieldName(dbOwner) + " AS " + getField_Simulation_Title().getName() + ", \n" +
         parameterSetsTable.getField_Name().getFullFieldName(dbOwner) + " AS " + getField_ParameterSet_Name().getName() + ", \n" +
         probeSetsTable.getField_Name().getFullFieldName(dbOwner) + " AS " + getField_ProbeSet_Name().getName() + ", \n" +
         simulationRunsAndResearchersView.getField_TruthValue().getFullFieldName(dbOwner) + " AS " + getField_TruthValue().getName() + "\n" +
     "FROM "  + summaryRuns + "\n" +
          "INNER JOIN " +  simulationRunsTable.getFullName(dbOwner) + " ON\n" + 
                summaryRunsView.getField_SimulationRun_PK().getFullFieldName(dbOwner) + " = " + simulationRunsTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
          "INNER JOIN " + simulationsTable.getFullName(dbOwner) + " ON\n" + 
                simulationRunsTable.getField_SimulationFK().getFullFieldName(dbOwner) + " = " + simulationsTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
          "INNER JOIN " + parameterSetsTable.getFullName(dbOwner) + " ON " +"\n" + 
                simulationRunsTable.getField_ParameterSetFK().getFullFieldName(dbOwner) + " = " + parameterSetsTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
          "INNER JOIN " + probeSetsTable.getFullName(dbOwner) + " ON\n" + 
                simulationRunsTable.getField_ProbeSetFK().getFullFieldName(dbOwner) + " = " + probeSetsTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
          "INNER JOIN " + researchersTable.getFullName(dbOwner) + " ON\n" + 
                simulationRunsTable.getField_ResearcherFK().getFullFieldName(dbOwner) + " = " + researchersTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
          "INNER JOIN " + simulationRunsAndResearchersView.getName() + " ON\n" + 
                simulationRunsTable.getField_PK().getFullFieldName(dbOwner) + " = " + simulationRunsAndResearchersView.getField_SimulationRun_PK().getFullFieldName(dbOwner);

  }
  

}
