
package abcm.db.components.views;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbView;
import abcm.db.components.tables.ABCMCodeModelsTable;
import abcm.db.components.tables.ABCMDataModelsTable;
import abcm.db.components.tables.ABCMResearchersTable;
import abcm.db.components.tables.ABCMSimulationsTable;

public abstract class ABCMSummarySimulationsAndDetailsView extends ABCMdbView {
  
  public ABCMSummarySimulationsAndDetailsView(String viewName, AbstractABCMDBGateway db){ 
    super(viewName, db);
  }

  public abstract ABCMdbField getField_Summary_PK();
  public abstract ABCMdbField getField_Simulation_PK();
  public abstract ABCMdbField getField_Simulation_Title();
  public abstract ABCMdbField getField_Simulation_Desc();
  public abstract ABCMdbField getField_CodeModel_Title();
  public abstract ABCMdbField getField_DataModel_Name();
  public abstract ABCMdbField getField_Simulation_DateCreated();
  public abstract ABCMdbField getField_Researcher_Name();
  public abstract ABCMdbField getField_TruthValue();
  
  public String getSQL(){
    String dbOwner = DB.getOwner();

    ABCMSummarySimulationsView summarySimulationsView = DB.getSummarySimulationsView();
    ABCMSimulationDetailsView simulationDetailsView = DB.getSimulationDetailsView();
    ABCMSimulationsTable simulationsTable = DB.getSimulationsTable();
    ABCMCodeModelsTable codeModelsTable = DB.getCodeModelsTable();
    ABCMResearchersTable researchersTable = DB.getResearchersTable();
    ABCMDataModelsTable dataModelsTable = DB.getDataModelsTable();

    String codeModels = codeModelsTable.getFullName(dbOwner);
    
    return
        "SELECT " +
            summarySimulationsView.getField_Summary_PK().getFullFieldName(dbOwner)     + " AS " + getField_Summary_PK().getName()             + ", \n" + 
            simulationsTable.getField_PK().getFullFieldName(dbOwner)                   + " AS " + getField_Simulation_PK().getName()          + ", \n" +
            simulationsTable.getField_Title().getFullFieldName(dbOwner)                + " AS " + getField_Simulation_Title().getName()       + ", \n" +
            simulationsTable.getField_Description().getFullFieldName(dbOwner)          + " AS " + getField_Simulation_Desc().getName()        + ", \n" +
            codeModelsTable.getField_Title().getFullFieldName(dbOwner)                 + " AS " + getField_CodeModel_Title().getName()        + ", \n" +
            dataModelsTable.getField_Name().getFullFieldName(dbOwner)                  + " AS " + getField_DataModel_Name().getName()         + ", \n" +
            simulationsTable.getField_DateCreated().getFullFieldName(dbOwner)          + " AS " + getField_Simulation_DateCreated().getName() + ", \n" +
            researchersTable.getField_ResearcherName().getFullFieldName(dbOwner)       + " AS " + getField_Researcher_Name().getName()        + ", \n" +
            simulationDetailsView.getField_TruthValue().getFullFieldName(dbOwner) + " AS " + getField_TruthValue().getName()             + "\n" +
       "FROM "  + summarySimulationsView.getName() + " \n" +
          "INNER JOIN " + simulationsTable.getFullName(dbOwner) + " ON\n" + 
               summarySimulationsView.getField_Simulation_PK().getFullFieldName(dbOwner) + " = " + simulationsTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
          "INNER JOIN " + codeModels + " ON\n" + 
               simulationsTable.getField_CodeModelFK().getFullFieldName(dbOwner) + " = " + codeModelsTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
          "INNER JOIN " + researchersTable.getFullName(dbOwner) + " ON\n" + 
               simulationsTable.getField_ResearcherFK().getFullFieldName(dbOwner) + " = " + researchersTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
          "INNER JOIN " + dataModelsTable.getFullName(dbOwner) + " ON\n" + 
               simulationsTable.getField_DataModelFK().getFullFieldName(dbOwner) + " = " + dataModelsTable.getField_PK().getFullFieldName(dbOwner) + "\n" +
          "INNER JOIN " + simulationDetailsView.getName() + " ON\n" + 
               simulationsTable.getField_PK().getFullFieldName(dbOwner) + " = " + simulationDetailsView.getField_Simulation_PK().getFullFieldName(dbOwner);
  }
  

}
