package abcm.db.managers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Vector;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbAbstractViewable;
import abcm.db.components.ABCMdbDatum;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbTable;
import abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager;
import abcm.utilities.XMLAssist;

/**
 * Manages the collection of Parameter Categories 
 * @author John T. Murphy
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class ParameterCategoriesManager extends AbstractDBStandardizedElementManager {

  /** Internal list of parameter categories  */
  private Vector parameterCategories = null;
  
  /**
   * Constructor
   * @param db Instance of an ABCM Gateway to provide database access.
   */
  public ParameterCategoriesManager(AbstractABCMDBGateway db){ super(db); }
  
  /* General */
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager#baseTable() 
   */
  protected ABCMdbTable baseTable(){                   return DB.getParameterCategoriesTable();                        }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager#baseTablePKField() 
   */
  protected ABCMdbField baseTablePKField(){            return DB.getParameterCategoriesTable().getField_PK();          }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager#baseTableNameField() 
   */
  protected ABCMdbField baseTableNameField() {         return DB.getParameterCategoriesTable().getField_Name();        }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager#baseTableDescriptionField() 
   */
  protected ABCMdbField baseTableDescriptionField(){   return DB.getParameterCategoriesTable().getField_Description(); }

  /* XML Creation (not implemented)*/
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBManager#recordsView() 
   */
  protected ABCMdbAbstractViewable recordsView() {     return DB.getParameterCategoriesTable();                        }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBManager#recordsViewPKField() 
   */
  protected ABCMdbField recordsViewPKField() {         return DB.getParameterCategoriesTable().getField_PK();          }
  
  
  /** 
   * Does nothing; XML representation is not implemented
   * @return null 
   */
  protected String xmlOuterNodeName() {              return null;     }
  
  /**
   * Does nothing; XML representation is not implemented
   * @param xml XMLAssist object constructing the xml document
   * @param rs result set containing records to be included
   * @param includeComments true if comments are to be included
   * @throws Exception generic exception
   */
  protected void addInstancesToXML(XMLAssist xml, ResultSet rs,
      boolean includeComments) throws Exception {  }

  /* HTML Creation (not implemented)*/
  
  /** 
   * Does nothing; no stylesheet is associated 
   * @return null
   */
  protected String getInstanceStylesheetName() {     return null;     }

  /** 
   * Does nothing; no stylesheet is associated
   * @return null 
   */
  protected String getListStylesheetName() {         return null;     }

  /* Saving (not implemented)*/
  
  /** 
   * Does nothing; parameter categories cannot be saved programmatically 
   */
  protected void clearInstanceToSave() {                              }

  /**
   * Does nothing; parameter categories cannot be saved programmatically
   * @return null
   */
  protected ABCMdbDatum datumToSave() {              return null;     }

  
  /* Special */
  
  /**
   * Gets the list of parameter categories
   * @return Vector of parameter categories
   */
  public Vector getParameterCategories(){
    if(parameterCategories == null) setParameterCategories();
    return parameterCategories;
  }    
  
  /**
   * Sets the internal vector of parameter categories. Memoized so
   * that this is retrieved only once.
   */
  private void setParameterCategories(){
    Vector paramCats = new Vector();
    try{
      String sql = DB.getParameterCategoriesTable().getSelectAllSQL(DB.getOwner());
      Connection con = DB.getGuestConnection();
      Statement st = con.createStatement();
      
      ResultSet rs = st.executeQuery(sql);
      
      while(rs.next()){
        paramCats.add(rs.getString(DB.getParameterCategoriesTable().getField_Name().getName()));        
      }
      
      rs.close();
      st.close();
      con.close();
      
    }
    catch(Exception e){
      addToErrorReport("Could not retrieve list of parameter categories: " + e.getMessage());
    }
    parameterCategories = paramCats;
    
  }
  
}
