package abcm.db.managers;

import java.sql.ResultSet;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbAbstractViewable;
import abcm.db.components.ABCMdbDatum;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbTable;
import abcm.db.managers.abstractparents.AbstractNonAuditableDBManager;
import abcm.utilities.XMLAssist;

/**
 * Database Manager for list of Truth Values
 *
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class TruthValueManager extends AbstractNonAuditableDBManager {
  
  /** See the ABCM documentation for details  */  public static final long CERTAIN         = 0;
  /** See the ABCM documentation for details  */  public static final long RECONSTRUCTED   = 1;
  /** See the ABCM documentation for details  */  public static final long INFERRED        = 2;
  /** See the ABCM documentation for details  */  public static final long LIKELY          = 3;
  /** See the ABCM documentation for details  */  public static final long POSSIBLE        = 4;
  /** See the ABCM documentation for details  */  public static final long IMPLAUSIBLE     = 5;
  /** See the ABCM documentation for details  */  public static final long IMPOSSIBLE      = 6;
  /** See the ABCM documentation for details  */  public static final long DEBUGGING       = 7;
  
  /** Description of this truth value to be saved   */  private String descriptionToSave;
  /** Value (text) of this truth value to be saved  */  private String truthValueToSave;
  
  /**
   * Constructor
   * @param db Instance of an ABCM Gateway to provide database access.
   */
  public TruthValueManager(AbstractABCMDBGateway db){ super(db); }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager#baseTable() 
   */
  protected ABCMdbTable baseTable(){                return DB.getTruthValuesTable();                          }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager#baseTablePKField() 
   */
  protected ABCMdbField baseTablePKField(){         return DB.getTruthValuesTable().getField_PK();            }  
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager#baseTableNameField() 
   */
  protected ABCMdbField baseTableNameField(){       return DB.getTruthValuesTable().getField_TruthValue();    }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBManager#recordsView() 
   */
  protected ABCMdbAbstractViewable recordsView(){   return DB.getTruthValuesTable();                          }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBManager#recordsViewPKField() 
   */
  protected ABCMdbField recordsViewPKField(){       return DB.getTruthValuesTable().getField_PK();            }

  
  /* Saving */
  
  /**
   * Determines if the instance to be saved is value
   * @return true if the instance to be saved has been set and is valid, false otherwise
   */
  public boolean instanceToSaveIsValid(){
    return ((descriptionToSave != null) &&
            (truthValueToSave  != null));
  }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBManager#clearInstanceToSave() 
   */
  protected void clearInstanceToSave() {
    descriptionToSave = null;
    truthValueToSave  = null;
  }

  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBManager#datumToSave() 
   */
  protected ABCMdbDatum datumToSave() {
    return (instanceToSaveIsValid() ?  DB.getTruthValuesTable().getDatum(descriptionToSave, truthValueToSave) :  null);
  }
  
  
  // Not Implemented
  /** 
   * Not used; overridden to take no action
   * @throws Exception generic exception 
   */
  protected void addInstancesToXML(XMLAssist xml, ResultSet rs, boolean includeComments) throws Exception {    }
  
  /** 
   * Not used; overridden to return null
   * @return null; 
   */
  protected String getInstanceStylesheetName() {    return null;    }

  /** 
   * Not used; overridden to return null
   * @return null; 
   */
  protected String getListStylesheetName() {        return null;    }

  /** 
   * Not used; overridden to return null
   * @return null; 
   */
  protected String xmlOuterNodeName() {             return null;    }

  
  
}
