package abcm.db.managers;

import java.sql.ResultSet;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbAbstractViewable;
import abcm.db.components.ABCMdbDatum;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbTable;
import abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager;
import abcm.utilities.XMLAssist;


/**
 * Database Manager for list of Work Types
 *
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class WorkTypeManager extends AbstractDBStandardizedElementManager {

  /**
   * Constructor
   * @param db Instance of an ABCM Gateway to provide database access.
   */
  public WorkTypeManager(AbstractABCMDBGateway db){    super(db);   }
  
  /* General */
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager#baseTable() 
   */
  protected ABCMdbTable baseTable(){                 return DB.getReferenceWorkTypesTable();                  }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager#baseTablePKField() 
   */
  protected ABCMdbField baseTablePKField(){          return DB.getReferenceWorkTypesTable().getField_PK();    }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBStandardizedElementManager#baseTableNameField() 
   */
  protected ABCMdbField baseTableNameField() {       return DB.getReferenceWorkTypesTable().getField_Type();  }
  
  /* XML Creation (not implemented)*/
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBManager#recordsView() 
   */
  protected ABCMdbAbstractViewable recordsView() {   return DB.getReferenceWorkTypesTable();                  }
  
  /** 
   * @see abcm.db.managers.abstractparents.AbstractDBManager#recordsViewPKField() 
   */
  protected ABCMdbField recordsViewPKField() {       return DB.getReferenceWorkTypesTable().getField_PK();    }

  /** 
   * Does not apply; not implemented
   * @return null 
   */
  protected String xmlOuterNodeName() {          return null;    }
  
  /** 
   * Does not apply; not implemented
   * @throws Exception generic exception
   */
  protected void addInstancesToXML(XMLAssist xml, ResultSet rs, boolean includeComments) throws Exception {   }


  /* HTML Creation (not implemented)*/
  /** 
   * No stylesheet applies
   * @return returns null 
   */
  protected String getInstanceStylesheetName() { return null;    }

  /** 
   * No stylesheet applies
   * @return returns null 
   */
  protected String getListStylesheetName() {     return null;    }

  /* Saving (not implemented)*/
  /** 
   * Not implemented; saving of this element is not permitted programmatically
   */
  protected void clearInstanceToSave() {   }

  /** 
   * Not implemented; saving of this element is not permitted programmatically
   * @return null 
   */
  protected ABCMdbDatum datumToSave() {          return null;    }
  
  
}

