package abcm.db.managers.abstractparents;

import abcm.db.AbstractABCMDBGateway;


/**
 * Database Manager for linked tables, for which the manager is an 'inner' manager.
 * Eliminates some functionality of the parent class for simplicity. 
 *
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public abstract class AbstractDBLinkTableInnerManager extends AbstractDBLinkTableManager {

  /** Containing class (another Manager class)  */
  private AbstractDBManager containerClass;
  
  /**
   * Constructor
   * @param db Instance of an ABCM Gateway to provide database access.
   * @param containedBy the containing class
   */
  public AbstractDBLinkTableInnerManager(AbstractABCMDBGateway db, AbstractDBManager containedBy){ 
    super(db); 
    containerClass = containedBy;
  }
  
  /** 
   * Returns null; no stylesheet is available 
   */
  protected final String getInstanceStylesheetName() {      return null;    }

  /** 
   * Returns null; no stylesheet is available 
   */
  protected final String getListStylesheetName() {          return null;    }

  /** 
   * returns null; no XML outer node is used 
   */
  protected final String xmlOuterNodeName() {               return null;    }


  /** 
   * This is overridden to redirect error reports to the containing class.
   * @param addendum text to be added to parent class's error report. 
   */
  public void addToErrorReport(String addendum){ containerClass.addToErrorReport(addendum); }
  
}
