package abcm.db.managers.abstractparents;

import abcm.db.AbstractABCMDBGateway;

/**
 * The superclasses of this object do not provide methods for deleting records from the database.
 * This omission is intentional: some objects must follow strict audit rules, while others
 * need not. This class provides simple deleting functionality for cases in which audit rules
 * are not required.
 * 
 * In general, implementing classes should simply provide a deleteItem(PK) method; this can
 * operate in any way, but most conveniently will invoke the deleteItem(ABCMdbTable, ABCMdbField, PK)
 * method provided here. 
 * 
 * @author John T. Murphy
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public abstract class AbstractNonAuditableDBManager extends AbstractDBStandardizedElementManager {

  /**
   * Constructor
   * @param db Instance of an ABCM Gateway to provide database access.
   */
  public AbstractNonAuditableDBManager(AbstractABCMDBGateway db){    super(db);    }

  /**
   * Deletes an item from the database in the base table being managed by this class
   * @param PK primary key of the item to be deleted
   * @return true if successful, false otherwise
   */
  public boolean deleteItem(long PK){ return DB.deleteItem(baseTable(), baseTablePKField(), PK); }
    
}
