package abcm.db.syntaxengines;

/**
 * A translator so that the db functions can work with different
 * variants of SQL syntax.
 * 
 * @author John T. Murphy
 * Apr 25, 2010
 */
public abstract class AbstractSQLSyntaxEngine implements SQLDataTypes{

  // Utility
  protected static String getFieldList(String[] fields){
    String fl = "";
    for(int i = 0; i < fields.length; i++){
      fl += fields[i] + (i < (fields.length - 1) ? ", " : "");
    }
    return fl;
  }
  
  public abstract String getDataTypeCreateRepesentation(SQLDataType dataType);
  
  public abstract String getPrimaryKeyIdentityFieldCreationKeywords();
  
  public abstract String getIdentityOnlyFieldCreationKeywords();

  public abstract String getCreateIndexStatement(String indexName, String tableName, String[] fields);
  
  public abstract String getForeignKeyCreateConstraintStatement(String tableName, 
      String FKName, String[] thisTableFields, String FKTable, String[] foreignFields);
  
//  public abstract String getBasicForeignKeyConstraint(String columnName, String foreignTableName, String foreignFieldName);
 
  public abstract String[] getCreateTableSQL(String dbOwner, String tableName, String[] fieldCreationRepresentations, 
      String[] multiFieldPKFieldList, String[] indexCreationRepresentations, String[] foreignKeyCreationRepresentations);
  
  public abstract String getRetrieveNamesOfAllTablesSQL();
  
  // Standard- override if a given implementation uses something very different
  /**
   * Generates a 'WHERE" SQL clause using the specified field name and a numeric value for the comparand.
   * @param field name of the field to be filtered
   * @param val value to be filtered
   * @return the text of the clause
   */
  public String where(String field, long val){   return "WHERE " + field + " = " + val;         }
  
  /**
   * Generates a 'WHERE" SQL clause using the specified field name and a String value for the comparand.
   * @param field name of the field to be filtered
   * @param val value to be filtered
   * @return the text of the clause
   */
  public String where(String field, String val){ return "WHERE " + field + " = '" + val + "'";  }
  
  /**
   * Generates an 'ORDER BY' SQL clause in the specified order (ascending or descending).
   * @param field field to be sorted
   * @param descending true if descending order is required, false otherwise
   * @return the text of the clause
   */
  public String orderBy(String field, boolean descending){ return "ORDER BY " + field + (descending ? " DESC" : ""); }
  

  /**
   * Simple default (assume Ascending)
   * @param field
   * @return
   */
  public final String orderBy(String field){return orderBy(field, true);}

  public abstract String getNullFunction();
  
  public abstract String getDottedName(String originalName);
  
  public abstract String getInsertFormat(SQLDataTypes.SQLDataType dataType, String value);
  
  public abstract String getDateTimeInsertFormat();
  
  public abstract String getOptimizerHint();
  
  public abstract String getBooleanTrueReturnString();
  
  
  
}
