package abcm.db.syntaxengines;

import java.util.Hashtable;
import java.util.Vector;


/**
 * An SQL Syntax Engine for MS-SQL Server 'Transact SQL'
 * @author John T. Murphy
 * Apr 7, 2010
 */
public class MSSQLServerSyntaxEngine extends AbstractSQLSyntaxEngine {
  
  private static final String MSSQLSERVERdateTimeInsertFormat = "MM/dd/yyyy HH:mm:ss";
  
  /**
   * @see abcm.db.syntaxengines.AbstractSQLSyntaxEngine#getDataTypeCreateRepesentation(abcm.db.syntaxengines.SQLDataTypes.SQLDataType)
   */
  public String getDataTypeCreateRepesentation(SQLDataType dataType){
    if     (dataType instanceof SQLDataTypes.SQLINTDataType) 
        return "int";
    else if(dataType instanceof SQLDataTypes.SQLVCHARDataType){
        return "varchar(" + ((SQLDataTypes.SQLVCHARDataType)dataType).getLength() + ")";
    }
    else if(dataType instanceof SQLDataTypes.SQLTEXTDataType) 
        return "text";
    else if(dataType instanceof SQLDataTypes.SQLDateTimeDataType) 
        return "datetime";
    else if(dataType instanceof SQLDataTypes.SQLFloatDataType)
        return "float(" + ((SQLDataTypes.SQLFloatDataType)dataType).getSize() + ")";
    else if(dataType instanceof SQLDataTypes.SQLBooleanDataType)
        return "bit";
    else if(dataType instanceof SQLDataTypes.SQLCHARDataType){
      return "char(" + ((SQLDataTypes.SQLCHARDataType)dataType).getLength() + ")";
    }
    else
        return "";
  }

  
  public String getPrimaryKeyIdentityFieldCreationKeywords(){
    return "IDENTITY (1,1) PRIMARY KEY NONCLUSTERED";
  }
  
  public String getIdentityOnlyFieldCreationKeywords(){
    return "IDENTITY (1,1)";
  }
  
  public String getCreateIndexStatement(String indexName, String tableName, String[] fieldList){
    return "CREATE UNIQUE NONCLUSTERED INDEX" + " " + indexName + " ON " + tableName + " (" + getFieldList(fieldList) + ")";
  }
  
//  public String getBasicForeignKeyConstraint(String columnName, String foreignTableName, String foreignFieldName){
//    return "FOREIGN KEY REFERENCES" + " " + foreignTableName + " (" + foreignFieldName + ")";
//  }
   
  public String[] getCreateTableSQL(String dbOwner, String tableName, String[] fieldCreationRepresentations, 
      String[] multiFieldPKFieldList, String[] indexCreationRepresentations, String[] foreignKeyCreationRepresentations){
    
    Vector<String> sqls = new Vector<String>();
    String sql = "";
    sql += "CREATE TABLE " + tableName + " (\n"; 
    for(int i = 0; i < fieldCreationRepresentations.length; i++){
      sql += "       " + fieldCreationRepresentations[i] + ((i < (fieldCreationRepresentations.length - 1)) ? ", " : "") + "\n";
    }
    
    if(multiFieldPKFieldList != null){
      String mfpk = "";
      for(int i = 0; i < multiFieldPKFieldList.length; i++){
        mfpk = mfpk + multiFieldPKFieldList[i] + (i < ((multiFieldPKFieldList.length - 1)) ? ", " : ""); 
      }
      sql += "\nCONSTRAINT " + tableName + "PK PRIMARY KEY NONCLUSTERED (" + mfpk + ")\n";
    }
    sql += ");";
    sqls.add(sql);
    
    // And collect the indexes; these will require separate SQL statements
    for(String ics: indexCreationRepresentations) sqls.add(ics + ";");
      
    // Foreign Keys also require separate SQL statements
    for(String fk: foreignKeyCreationRepresentations) sqls.add(fk + ";");
        
    String[] ret = new String[sqls.size()];
    for(int i = 0; i < ret.length; i++) ret[i]= sqls.get(i);
    return ret;  
    
  }

  public String getRetrieveNamesOfAllTablesSQL(){
    return "select table_name as Name from INFORMATION_SCHEMA.Tables where TABLE_TYPE ='BASE TABLE'";
    //return "SELECT * FROM sys.Tables;";
    
  }
  
  public String getForeignKeyCreateConstraintStatement(String tableName, 
      String FKName, String[] thisTableFields, String FKTable, String[] foreignFields){
    return "ALTER TABLE " + tableName + " ADD CONSTRAINT " + FKName + " " +
      "FOREIGN KEY (" + getFieldList(thisTableFields) + ") " +
      "REFERENCES " + FKTable + " (" + getFieldList(foreignFields) + ")";
  }
  
  public String[] getCreateTableSQL(String dbOwner, Vector orderedFieldList, String multiFieldPKFieldList, Hashtable indexList, Hashtable FKList){
    return new String[1];
  }
  
  public String getNullFunction(){ return "ISNULL"; }
  
  public String getDottedName(String originalName){
    return "[" + originalName.replaceAll("\\Q.", "].[") + "]";
  }
  
  public String getInsertFormat(SQLDataTypes.SQLDataType dataType, String value){
    return "'" + value + "'";
  }
  
  public String getDateTimeInsertFormat(){ return MSSQLSERVERdateTimeInsertFormat; }
  
  public String getOptimizerHint(){ return ""; }
  
  public String getBooleanTrueReturnString(){ return "1"; }
  
}

