package abcm.db.syntaxengines;

import java.util.Vector;


/**
 * @author John T. Murphy
 * Apr 7, 2010
 */
public class MySQLSyntaxEngine extends AbstractSQLSyntaxEngine{

  private static final String MySQLdateTimeInsertFormat = "yyyy-MM-dd HH:mm:ss";
  
  /**
   * @see abcm.db.syntaxengines.AbstractSQLSyntaxEngine#getDataTypeCreateRepesentation(abcm.db.syntaxengines.SQLDataTypes.SQLDataType)
   */
  public String getDataTypeCreateRepesentation(SQLDataType dataType){
    if     (dataType instanceof SQLDataTypes.SQLINTDataType) 
        return "INT";
    else if(dataType instanceof SQLDataTypes.SQLVCHARDataType){
        return "VARCHAR(" + ((SQLDataTypes.SQLVCHARDataType)dataType).getLength() + ")";
    }
    else if(dataType instanceof SQLDataTypes.SQLTEXTDataType) 
        return "TEXT";
    else if(dataType instanceof SQLDataTypes.SQLDateTimeDataType) 
        return "DATETIME";
    else if(dataType instanceof SQLDataTypes.SQLFloatDataType)
        return "DOUBLE";//(" + ((SQLDataTypes.SQLFloatDataType)dataType).getSize() + ")";
    else if(dataType instanceof SQLDataTypes.SQLBooleanDataType)
        return "BOOL";//BIT(1)";
    else if(dataType instanceof SQLDataTypes.SQLCHARDataType){
      return "CHAR(" + ((SQLDataTypes.SQLCHARDataType)dataType).getLength() + ")";
    }
    else
        return "";
  }
  
  public String getPrimaryKeyIdentityFieldCreationKeywords(){
    return "AUTO_INCREMENT PRIMARY KEY";
  }
  
  public String getIdentityOnlyFieldCreationKeywords(){
    return "UNIQUE";
  }

  public String getCreateIndexStatement(String indexName, String tableName, String[] fieldList){
    return "CREATE UNIQUE INDEX" + " " + indexName + " ON " + tableName + " (" + getFieldList(fieldList) + ")";
  }
  
//  public String getBasicForeignKeyConstraint(String columnName, String foreignTableName, String foreignFieldName, boolean allowNulls){
//    return "FOREIGN KEY (" + columnName + ") REFERENCES" + " " + foreignTableName + " (" + foreignFieldName + ") ";
//  }

  public String getForeignKeyCreateConstraintStatement(String tableName, 
      String FKName, String[] thisTableFields, String FKTable, String[] foreignFields){
    return "ALTER TABLE " + tableName + " ADD CONSTRAINT " + FKName + " " +
      "FOREIGN KEY (" + getFieldList(thisTableFields) + ") " +
      "REFERENCES " + FKTable + " (" + getFieldList(foreignFields) + ")";
  }

  
  public String[] getCreateTableSQL(String dbOwner, String tableName, String[] fieldCreationRepresentations, 
      String[] multiFieldPKFieldList, String[] indexCreationRepresentations, String[] foreignKeyCreationRepresentations){

    Vector<String> sqls = new Vector<String>();
    String sql = "";
    sql += "CREATE TABLE " + tableName + " (\n"; 
    for(int i = 0; i < fieldCreationRepresentations.length; i++){
      sql += "       " + fieldCreationRepresentations[i] + ((i < (fieldCreationRepresentations.length - 1)) ? ", " : "") + "\n";
    }
    
    if(multiFieldPKFieldList != null){
      String mfpk = "";
      for(int i = 0; i < multiFieldPKFieldList.length; i++){
        mfpk = mfpk + multiFieldPKFieldList[i] + (i < ((multiFieldPKFieldList.length - 1)) ? ", " : ""); 
      }
      sql += ",\nPRIMARY KEY (" + mfpk + ")\n"; // Note initial comma
    }
    
    sql += ");";
    sqls.add(sql);
    
    // And collect the indexes; these will require separate SQL statements
    for(String ics: indexCreationRepresentations) sqls.add(ics);
      
    // Foreign Keys require separate SQL statements
    for(String fk: foreignKeyCreationRepresentations) sqls.add(fk);
        
    String[] ret = new String[sqls.size()];
    for(int i = 0; i < ret.length; i++)  ret[i]= sqls.get(i);
    return ret;  
  
  }

  public String getRetrieveNamesOfAllTablesSQL(){
    return "SHOW FULL TABLES WHERE TABLE_TYPE = 'BASE TABLE';";
  }
  
  public String getNullFunction(){ return "IFNULL"; }    
  
  public String getDottedName(String originalName){
    return originalName;
  }
  
  public String getInsertFormat(SQLDataTypes.SQLDataType dataType, String value){
    return ((dataType instanceof SQLDataTypes.SQLBooleanDataType) ?  value : "'" + value + "'");
  }
  
  public String getDateTimeInsertFormat(){ return MySQLdateTimeInsertFormat; } 

  public String getOptimizerHint(){ return "STRAIGHT_JOIN"; }
  
  public String getBooleanTrueReturnString(){ return "1"; }
}
