package abcm.graphics;

import java.awt.Color;
import java.util.Vector;

/**
 * A Color Sequence is a helper object that lists
 * a series of colors that can be used in a sequence
 * when color differentiation is needed; for example,
 * in a graph in which the first line should be red,
 * the second blue, the third green, etc., so long as
 * there are no repeated colors.<br/><br/>
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class ColorSequence {

  /**
   * The collection of colors represented by this sequence.
   */
  private Vector colors;
  
  /**
   * Generic constructor that initializes
   * the sequence.
   */
  public ColorSequence(){
    colors = new Vector();
    colors.add(new Color(255,   0,   0));
    colors.add(new Color(  0,   0, 255));
    colors.add(new Color(  0, 255,   0));
    colors.add(new Color(255, 255,   0));
    colors.add(new Color(128,   0, 255));
    colors.add(new Color(  0, 128, 255));
    colors.add(new Color(128,   0,   0));
    colors.add(new Color(  0, 128,   0));
    colors.add(new Color(  0,   0, 128));
    colors.add(new Color(128, 128,   0));
    colors.add(new Color(128,   0, 128));
    colors.add(new Color(  0, 128, 128));
    colors.add(new  Color(255, 128,  0));
    colors.add(new Color(255,   0, 128));
    colors.add(new Color(128, 255,   0));
    colors.add(new Color(  0, 255, 128));
  }
  
  /**
   * Get the color, given this sequence.
   * @param sequenceNum the sequence number
   * @return the color indicated by this sequence number
   */
  public Color getColor(int sequenceNum){
    if((sequenceNum < 0 ) || (sequenceNum >= colors.size())) return new Color(0,0,0);
    return (Color)colors.get(sequenceNum);
  }
 
}
