package abcm.graphics;

/**
 * All objects that can be depicted in an SVG representation should implement
 * this interface. They can then be passed to SVGAssist and added to an SVG document.
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public interface SVGRepresentable {

  /**
   * Any object that implements the SVGRepresentable interface should be drawable by
   * an SVG Assist object that will invoke this method to cause the SVGRepresentable object
   * to add itself to the SVG document using the information about scale, etc.,
   * obtainable in the SVG Assist object.  
   * Sometimes an object must retrieve information about the document within which it
   * is being drawn. For example, a simple SVGRepresentable might draw a 1 inch circle
   * (or a font of a particular size). However, when it is nested in other SVGRepresentables,
   * it cannot easily know what coordinate scale to use to achieve a 1 inch circle. This is
   * by design in SVG, but it is inconvenient for some uses. In the SVGRepresentable scheme,
   * the 'draw' method passes an object that permits access to the current scale information.
   * @param svg
   */
  public void addToSVG(SVGAssist svg);
  
}
