package abcm.graphics;

import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import abcm.math.Point;
import abcm.math.Polygon;

/**
 * Creates an x3d 'IndexedFaceSet' from a given list of polygons.
 * 
 * Use of this class is two-step: the constructor is invoked with a list of
 * polygons to be used to create the IndexedFaceSet. Once constructed,
 * the object can return the point list and face list needed to
 * create the IndexedFaceSet x3d.
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class X3DIndexedFaceSetCreator {

  /** List of points in the Indexed Face Set, in proper X3D format  */
  private String pointList = "";
  /** List of faces in the Indexed Face Set, in proper X3D format  */
  private String faceList  = "";
  
  /**
   * Constructor. The actual creation of the indexed face set is done here.
   * @param polygons the list of polygons to be made into the IndexedFaceSet object
   */
  public X3DIndexedFaceSetCreator(List polygons){
        
    Hashtable Vertices = new Hashtable();
    Hashtable Values = new Hashtable();
    
    int c = 0;
    for (int i = 0; i < polygons.size(); i++) {
      Polygon P = (Polygon) polygons.get(i);
      P.makeCounterClockwise();
//      P.removeDuplicatePoints();
      Vector points = P.getPoints();
      for (int x = 0; x < points.size(); x++) {
        Point p = (Point) points.get(x);
        // See if this point has already been added
        if (!Vertices.containsKey(p.coordString())) {
          Integer I = new Integer(c);
          Vertices.put(p.coordString(), I);
          Values.put(I, p);
          c++;
        }
      }
    }

    for (int i = 0; i < c; i++) {
      Point p = (Point) Values.get(new Integer(i));
      if (i > 0) pointList += ",\n ";
      pointList += p.getX() + ", " + p.getZ() + ", " + p.getY();
    }

    for (int i = 0; i < polygons.size(); i++) {
      Polygon P = (Polygon) polygons.get(i);
      Vector points = P.getPoints();
      if (i > 0) faceList += ",\n ";
      String termString = "";
      for (int x = points.size() - 1; x >= 0; x--) {
        Point p = (Point) points.get(x);
        if (x == (points.size() - 1))
          termString = ""
              + ((Integer) Vertices.get(p.coordString())).intValue();
        faceList += ((Integer) Vertices.get(p.coordString())).intValue() + ", ";
      }
      faceList += termString + ", -1";
    }
    
  }
  
  /**
   * Gets the point list for the IndexedFaceSet being created.
   * @return the point list. The format is appropriate for defining in x3d
   */
  public String getPointList(){ return pointList; }
  
  /**
   * Gets the face list for the IndexedFaceSet being created
   * @return the face list. The format is appropriate for defining in x3d
   */
  public String getFaceList(){  return faceList;  }
  
}
