package abcm.math;

import java.util.Random;

/**
 * This class wraps a random number generator object;
 * it contains a method that returns a random double
 * 0 <= x < 1. The constructor allows you to specify
 * the kind of random number generator to be used; many
 * different kinds may be 'plugged in' to this and available
 * as options to the user.
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class BasicRNG {

/* Public constants */
/** Default random number generator identifier = 0 */
public static final int DEFAULT_JAVA_RNG = 0;


/* Private instance variables */
/** Random number generator type in use */
private int type;

/** Seed used when the random number generator was initialized */
private long seed;
 
/** Random number generator */
private Random random;

  /**
   * Generic constructor with no arguments.
   * Initializes to default random number generator.
   */
  public BasicRNG(){
    type = DEFAULT_JAVA_RNG;
  }

  /**
   * Constructor that allows the user to specify the Random
   * Number Generator Type to be used
   * @param type type of RNG to be used
   */
  public BasicRNG(int type){
    this(type, System.currentTimeMillis());
  }
  
  /**
   * Constructor
   * @param type switch for type of random number generator to use
   * @param seed seed to be used when random number generator is initialized
   */
  public BasicRNG(int type, long seed){
    this.type = type;
    this.seed = seed;
    switch(type){
       case DEFAULT_JAVA_RNG:
       default:
         random = new Random(this.seed);
         break;
    }
  }
  
  /**
   * Returns the next random number (0 < x < 1) from the ABCM Random Number
   * Generator, whichever RNG underlies this.
   * @return a random number between zero and one
   */
  public double nextDouble(){
    switch(type){
      case DEFAULT_JAVA_RNG:   return random.nextDouble();
      default:                 return 0;
    }
  }

}
