package abcm.math;

/**
 * An object that can be set to include a certain number of days per month.
 * Object also includes some functions for getting details about the month.
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class Month {

  /** 
   * A name for this month
   */         
  private String monthName;
  
  /** 
   * The number of days in this month  
   */         
  private int    monthDays;
  
  /** 
   * The day of the year on which the first day of this month falls  
   */         
  private int    monthFirstDay;
  
  /**
   * Constructs and instance of the Month object with the given name, the
   * specified number of days, and the day of the year on which the
   * first day of the month will be considered to fall.
   * @param Name a name for this month
   * @param Days the number of days in this month
   * @param firstDay the day of the year on which the first day of this month falls
   */ 
  public Month(String Name, int Days, int firstDay){
    monthName     = Name;
    monthDays     = Days;
    monthFirstDay = firstDay;
  }
  
  /**
   * Gets the name of this month.
   * @return the name of this instance of the month object.
   */
  public String getName()      {    return monthName;                 }
  
  /**
   * Gets the abbreviation for this month.
   * This is always the first three letters of the month name- no attempt is made to make
   * abbreviations unique or independent of the name of the month. In other words, it is
   * simply a convenience for prettifier routines.
   * @return the first three letters of the month name.
   */
  public String getABB()       {    return monthName.substring(0,3).toUpperCase();  }
  
  /**
   * Gets the number of days in this month.
   * @return the number of days in this month.
   */
  public int getDays()         {    return monthDays;                 }
  
  /**
   * Gets the day of the year on which the first day of this month
   * falls.
   * @return the day of the year on which the first day of this month falls.
   */
  public int getMonthFirstDay(){    return monthFirstDay;             }
  
}
