package abcm.math;

import java.util.HashMap;
import java.util.Vector;

/**
 * A factory that accepts a Point object and
 * returns either that object, if it is unique, or
 * any previously accepted Point object that is so close
 * as to be virtually identical, within a specified precision.
 *
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class PointStore{
  /** Default precision for Point Store */
  public static final int DEFAULT_PRECISION = 6;
  
  /** Collection of points in this store */
  private Vector points = new Vector();
  
  /** Collection of points indexed by name */
  private HashMap pointsSet = new HashMap();
  
  /** Precision to be used to establish when a new point is needed */
  private int precision = DEFAULT_PRECISION;
  
  /** 
   * Private representation of error value in points; uses precision as power of ten
   * for error value
   */
  private double epsilon;
  
  
  /**
   * Constructor
   */
  public PointStore(){
    this(DEFAULT_PRECISION);
  }
  
  /**
   * Constructor using specified precision
   * @param newPrecision
   */
  public PointStore(int newPrecision){
    precision = newPrecision;
    calculateEpsilon();
  }
  
  /**
   * Calculates epsilon given a precision value
   */
  private void calculateEpsilon(){
    epsilon = Math.pow(10, -1 * precision);
  }
  
  /**
   * Gets a new point from the store; if an existing point
   * will suffice, uses it, but if not adds the new point to the store
   * @param newPoint new point to compare
   * @return the new point or an existing, equivalent one
   */
  public Point getPoint(Point newPoint){
    String test = newPoint.coordString(precision);
    if(pointsSet.containsKey(test)) return (Point)pointsSet.get(test);
    pointsSet.put(test, newPoint);
    return newPoint;
    
    // Long way- not necessary...
//    for(int i = 0; i < points.size(); i++){
//      Point storedPoint = (Point)points.get(i);
//      if(storedPoint.coordString(precision).compareTo(newPoint.coordString(precision)) == 0) return storedPoint;
//    }
//    points.add(newPoint);
//    return newPoint;
    
  }
}
    