package abcm.model;

/**
 * An ABCM result is an object that contains one or both of a text and a numeric value,
 * and a boolean 'success' flag. It stands as a convenient means to return values
 * from some called function: the return can contain not only the result (in numeric or text form) 
 * but also an indication of whether that result is a successful one or whether some error
 * occurred.
 *  
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class ABCMResult {

  /** Text  */
  public String  text     = "";
  /** Numeric Value  */
  public double  value    = 0;
  /** Object */
  public Object object = null;
   
  /** Success flag  */
  public boolean success;
   
  /**
   * Constructor
   * @param Text text
   * @param Success success flag
   */
  public ABCMResult(String Text, boolean Success){
    text    = Text;
    success = Success;
  }
   
  /**
   * Constructor
   * @param val numeric value
   * @param Success success flag
   */
  public ABCMResult(double val, boolean Success){
    value   = val;
    success = Success;
    text    = Double.toString(val);  // This way the text will reflect the value, too
  }
   
  /**
   * Constructor
   * @param Text text
   * @param val numeric value
   * @param Success success flag
   */
  public ABCMResult(String Text, double val, boolean Success){
    text    = Text;
    value   = val;
    success = Success;     
  }

  /**
   * Constructor
   * @param Text
   * @param o
   * @param Success
   */
  public ABCMResult(String Text, Object o, boolean Success){
    text    = Text;
    object  = o;
    success = Success;
  }
  
  /**
   * For use in shortcut output situations, esp. debugging.
   * @see java.lang.Object#toString()
   */
  public String toString(){
    return (text + " (" + value + ")");
  }
  
}
