package abcm.model;

/**
 * A <code>ParseAction</code> is an abstract class that is instantiated to add
 * actions to the parser. An instance of a subclass of ParseAction can be
 * added to any subclass of Parser, and the resulting action will be executed
 * when the appropriate request is issued to the Parser object.
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public abstract class ParseAction {
  
  /**
   * Execute the specified command within the specified environment and return the result
   * @param command the command to be executed; the zeroth element should be the command and the remainder
   * are its arguments
   * @param env the environment in which this command is to be executed
   * @return the result of executing the command
   */
  public abstract ABCMResult executeCommand(String[] command, AbstractABCModelEnvironment env);
  
}
