package abcm.sim;

import abcm.cloning.ABCMDeepCloneable;


/**
 * A Configuration is a collection of static details
 * and values that are used throughout the course of
 * a simulation. As of the initial version, a
 * Configuration includes a list of plants and varieties,
 * along with the values for their relevant variables.<br/><br/> 
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public abstract class Configuration implements ABCMDeepCloneable {

  /* Instance Variables */
  /** Name of this Configuration  */
  private String name;
  
  /**
   * Creates a new configuration with the specified name
   * @param Name new name for this configuration
   */
  public Configuration(String Name){   name = Name.toUpperCase();  }
  
  /**
   * Gets the name of this Configuration
   * @return the name of this Configuration
   */
  public String getName(){     return name;     }

  /**
   * Gets a report describing the contents of this configuration. Specific child implementations
   * will differ in the content of this report
   * @return a report on this configuration
   */
  public abstract String getReport();
   
}
