package abcm.sim;

import abcm.cloning.ABCMCloneStore;
import abcm.cloning.ABCMDeepCloneable;


/**
 * A Data Model is a collection that includes a History and
 * a Configuration. A Data Model is identified by a name and 
 * provided with a description.<br/><br/>
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class DataModel implements ABCMDeepCloneable{
  
  /** PK of History contained in this Data Model  */
  private long   HistoryPK;
  /** PK of Configuration contained in this Data Model  */
  private long   ConfigurationPK;
  /** Name of this Data Model  */
  private String name;
  /** Description of this Data Model  */
  private String desc;
  
  /**
   * Creates a data model using the parameters specified
   * @param dmName new name for a data model
   * @param description description for this data model
   * @param histPK primary key of a History as stored in the HWM Database 
   * @param configPK primary key of a Configuration as stored in the HWM Database 
   */
  public DataModel(String dmName, String description, long histPK, long configPK){
    HistoryPK       = histPK;
    ConfigurationPK = configPK;
    name            = dmName;
    desc            = description;    
  }
  
  /**
   * Gets the primary key of the Configuration in this Data Model.
   * @return primary key of the Configuration
   */
  public long getConfigurationPK() {    return ConfigurationPK;    }
  

  /**
   * Gets the primary key of the History in this Data Model.
   * @return primary key of the History
   */
  public long getHistoryPK() {    return HistoryPK;  }
  
  
  /**
   * Gets the description for this Data Model
   * @return the Data Model's description
   */
  public String getDesc() {    return desc;  }
  
  /**
   * Gets the name of this Data Model.
   * @return the Data Model name
   */
  public String getName() {    return name;  }
  
  /**
   * Gets a clone of this object
   * @param cloneStore the Clone Store to be used
   * @return the cloned object
   */
  public ABCMDeepCloneable getDeepClone(ABCMCloneStore cloneStore){
    DataModel ret = new DataModel(name, desc, HistoryPK, ConfigurationPK);
    cloneStore.put(this, ret);
    return ret;
  }
}
