package abcm.sim;

import abcm.cloning.ABCMCloneStore;
import abcm.cloning.ABCMDeepCloneable;


/**
 * A HistoryInstruction is a simple object that includes a command, an
 * instruction order, a narrative order, an array of string arguments,
 * and a date.<br/><br/>
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class HistoryInstruction implements ABCMDeepCloneable{

  /** (Maximum) number of arguments per command  */
  public static final int NumCommandArgs = 10;
  
  /**
   * The primary key of the command, as stored in the ValidNarrativeCommands table
   * of the HWM Database
   */
  public int CommandID;
  
  /**
   * A list of arguments to this command, as an array of Strings
   */
  public String[] argList = new String[NumCommandArgs];
  
  /**
   * A number indicating the order of this HistoryInstructions in a series of commands
   */
  public double instructionOrder;
  
  /**
   * A number indicating the order of the Narrative from which this instruction
   * is drawn, given that a single History may have multiple Narratives that are
   * prioritized.
   */
  public int narrativeOrder;
  
  /**
   * The Simulation Date on which this HistoryInstruction is to be processed.
   */
  public double date;
  
  /**
   * The name of the Narrative from which this instruction is drawn.
   */
  public String narrativeName;
  

  /** 
   * @see abcm.cloning.ABCMDeepCloneable#getDeepClone(abcm.cloning.ABCMCloneStore) 
   */
  public ABCMDeepCloneable getDeepClone(ABCMCloneStore cloneStore){
    HistoryInstruction newHistoryInstruction = new HistoryInstruction();
    cloneStore.put(this, newHistoryInstruction);
    newHistoryInstruction.argList = new String[argList.length];
    for(int i = 0; i < argList.length; i++)  newHistoryInstruction.argList[i] = argList[i];
    newHistoryInstruction.CommandID        = this.CommandID;
    newHistoryInstruction.date             = this.date;
    newHistoryInstruction.instructionOrder = this.instructionOrder;
    newHistoryInstruction.narrativeName    = this.narrativeName;
    newHistoryInstruction.narrativeOrder   = this.narrativeOrder;
    return newHistoryInstruction;
  }
  
}
