package abcm.sim;

import java.util.Date;

import abcm.cloning.ABCMCloneStore;
import abcm.cloning.ABCMDeepCloneable;

/**
 * A Simulation is the specification of a Code Model and a Data Model.
 * 
 * Simulations can be loaded in a model or stored in the HWM Database; if the
 * latter, the Simulation will have a name and descriptors.
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class Simulation implements ABCMDeepCloneable{

  /** Name of this Simulation  */
  private String    name;
  /** Date this Simulation was created  */
  private Date      dateCreated;
  /** String indicating who created this Simulation  */
  private String    createdBy;
  /** PK of the Code Model used by this Simulation  */
  private long      codeModelPK = 0;
  /** PK of the Data Model used by this Simulation  */
  private long      dataModelPK = 0;
  
  /**
   * Constructor that supplies values for all the fields in a Simulation
   * object
   * @param simName Name
   * @param simDateCreated Date Created
   * @param simCreatedBy By whom the Simulation was created
   * @param simCodeModelPK PK of Code Model
   * @param simDataModelPK PK of Data Model
   */
  public Simulation(String simName, 
      Date simDateCreated, String simCreatedBy, 
      long simCodeModelPK, long simDataModelPK){
  
    name           = simName;
    dateCreated    = simDateCreated;
    createdBy      = simCreatedBy;
    codeModelPK    = simCodeModelPK;
    dataModelPK    = simDataModelPK;
    
  }
  
  /**
   * Gets the primary key of the Code Model
   * @return Returns the codeModelPK.
   */
  public long getCodeModelPK() {
    return codeModelPK;
  }
  
  /**
   * Gets the primary key of the Data Model
   * @return Returns the dataModelPK.
   */
  public long getDataModelPK() {
    return dataModelPK;
  }
  
  /**
   * Gets the name of this simulation
   * @return the simulation name
   */
  public String getSimName(){
    return name;
  }

  /** 
   * @see abcm.cloning.ABCMDeepCloneable#getDeepClone(abcm.cloning.ABCMCloneStore) 
   */
  public ABCMDeepCloneable getDeepClone(ABCMCloneStore cloneStore){
    Simulation ret = new Simulation(name, dateCreated, createdBy, codeModelPK, dataModelPK);
    cloneStore.put(this, ret);
    return ret;
  }
}
