package abcm.space;

import abcm.graphics.SVGAssist;
import abcm.graphics.SVGRepresentable;
import abcm.math.LineSegment;
import abcm.math.Point;

/**
 * A ContourLine is a LineSegment with two special characteristics: first,
 * it is SVGRepresentable, and so can easily be depicted in an SVG page;
 * second, it includes an additional instance variable that indicates elevation.
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
class ContourLine extends LineSegment implements SVGRepresentable{

  /* Instance Variables */
  /** Elevation  */
  private double val;
  
  
  /* Constructors */
  /**
   * Basic constructor.
   * @param P1 endpoint 1
   * @param P2 endpoint 2
   * @param altitude elevation
   * 
   */
  ContourLine(Point P1, Point P2, double altitude){
     super(P1, P2);
     val = altitude;
  }
  
  /* Graphic Output */
  /**
   * Adds this to an SVG document
   * @param svg the SVGAssist object within which this object is to be depicted
   */
  public void addToSVG(SVGAssist svg){
      

    String stroke = "green";
    double stroke_width = (1d/100d) * svg.getUnitsPerInchX();
    String otherTags = "id=\"" + super.IDString + "\" onclick=\"showContourInterval(evt, '" + super.IDString + "');\"";
    svg.addLine(P1.getX(), P1.getY(), P2.getX(), P2.getY(), stroke, stroke_width, otherTags);

    // Adding a label to the contour line- not tested
    
//    double contourLabelHeight = .2;
//    double contourLabelWidth  = .5;
//    double contourLabelBorderWidth = .01;
//    double contourLabelFont   = 8;
//    String fill   = "none";
//    xml.addLine("line",  "x1=\"" + P1.getX() + "\" " +
//                         "y1=\"" + P1.getY() + "\" " +
//                         "x2=\"" + P2.getX() + "\" " +
//                         "y2=\"" + P2.getY() + "\" " +
//                         "fill=\"" + fill + "\" " +
//                         "stroke=\"" + stroke +"\" " +
//                         "stroke-width=\"" + stroke_width + "\" " +
//                         otherTags,"");
//    xml.open("g","visibility=\"hidden\" " +
//             "id=\"contourPopup_" + this.IDString + "\" " +
//             "onclick=\"hideSelf(evt,'contourPopup_" + this.IDString + "');\"");
//      String outText = TextFormatter.roundWithUnits(val, 2, "m");
//      double pt = 8;
//      double yTextSize = mv.getTextHeight(outText, pt);
//      double xTextSize = mv.getTextWidth(outText, pt);
//      double xPos = this.P1.getX();
//      double yPos = this.P1.getY();
//      String tGroup = mv.makeText(0, 0, pt, outText, mv.getMetersPerInchX(), mv.getMetersPerInchY());
//      xml.addBlock(mv.getMapPopup(xPos, yPos, 
//             contourLabelWidth, contourLabelHeight,
//             contourLabelWidth * mv.getMetersPerInchX()/xTextSize, contourLabelHeight * mv.getMetersPerInchY(),
//             -.5 * contourLabelWidth, 0,
//             mv.makeText(0,0,contourLabelFont,outText, mv.getMetersPerInchX(), mv.getMetersPerInchY()), "black", contourLabelBorderWidth));
//    xml.closeTag();
  }      
  
  /**
   * Returns the elevation value of this contour line
   * @return the elevation of this contour line
   */
  public double getVal(){return val;}
}