package abcm.space;

/**
 * Single object that represents the volume difference between two
 * triangles. The two triangles are assumed to be in a relationship
 * to one another such that one is by definition 'above' the other,
 * even if their absolute coordinates do not appear to agree with this
 * definition. The result is that the volume difference is represented
 * by two numbers, both positive, considered to be the volume of A above B
 * and the volume of A below B.
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class TopographicVolumeDifferential {

  /** Volume of A below B  */
  protected double volumeBelow = 0;
  /** Volume of A above B  */
  protected double volumeAbove = 0;
  
  /**
   * Constructor (assumes values are zero)
   */
  public TopographicVolumeDifferential(){ }
  
  /**
   * Constructor
   * @param volumeAbove value to be used for volume above
   * @param volumeBelow value to be used for volume below
   */
  public TopographicVolumeDifferential(double volumeAbove, double volumeBelow){
    this.volumeAbove = volumeAbove;
    this.volumeBelow = volumeBelow;
  }
  
  /**
   * Adds an amount to the volume above
   * @param volumeToAdd amount to be added
   */
  public void addToVolumeAbove(double volumeToAdd){
    volumeAbove += volumeToAdd;
  }
  
  /**
   * Adds an amount to the volume below
   * @param volumeToAdd amount to be added
   */
  public void addToVolumeBelow(double volumeToAdd){
    volumeBelow += volumeToAdd;
  }
  
  /**
   * Adds the values from the specified topographic volume differential to
   * this one.
   * @param other the addend
   */
  public void add(TopographicVolumeDifferential other){
    this.volumeAbove += other.volumeAbove;
    this.volumeBelow += other.volumeBelow;
  }
  
  /**
   * @return the volume above portion of the TopographicVolumeDifferential
   */
  public double getVolumeAbove(){   return volumeAbove;   }
  
  /**
   * @return the volume below portion of the TopographicVolumeDifferential
   */
  public double getVolumeBelow(){   return volumeBelow;   }
   
}
