package abcm.utilities;

import java.io.InputStream;
import java.util.Properties;

import org.apache.log4j.helpers.Loader;

/**
 * AbstractABCMPropertyReader is a simple class that can be extended to read a
 * properties file from the required location. 
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public abstract class AbstractABCMPropertyReader {
    
  /**
   * Gets the specified property from the specified file
   * @param propertyID the property to be recovered
   * @param fileName the property file name to be interrogated
   * @return the value of the specified property, or an error message
   * indicating the failure of the operation.
   */
  protected static String getPropertyValue(String propertyID, String fileName){  
    Properties p = new Properties ();
    try{
      if(Loader.getResource(fileName) == null) System.out.println("Unable to find property file '" + fileName + "'");
      InputStream inputStream = Loader.getResource (fileName).openStream();
      if ( null != inputStream )    p.load (inputStream);      
    }
    catch(Exception E){
      return "Error getting property '" + propertyID + "': " + E.getLocalizedMessage();
    }
    String s = p.getProperty(propertyID);
    return(s != null ? s : "property '" + propertyID + "' not found in file '" + fileName + "'");
  } 
 
}
