package abcm.web;

import java.io.File;
import java.io.FileOutputStream;

import abcm.graphics.X3DAssist;
import abcm.graphics.X3DRepresentable;

/**
 * Creates and saves an x3d file based on a list of x3d representable objects that should be included
 * and a directory that specifies a file output location.
 *  
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class X3DViewer {
 
  /**
   * Creates the x3d file from the specified list of x3d representable objects. Saves the
   * resulting file to the specified directory
   * @param x3d_objects collection of objects to be included
   * @param imageOutputLocation directory to which output should be saved
   * @return the name of the file created
   */
  public String getX3D(X3DRepresentable[] x3d_objects, String imageOutputLocation){
    return getX3D(x3d_objects, imageOutputLocation, "");
  }
  
  
  /**
   * Creates the x3d file from the specified list of x3d representable objects. Saves the
   * resulting file to the specified directory
   * @param x3d_objects collection of objects to be included
   * @param imageOutputLocation directory to which output should be saved
   * @param filePrefix prefix to be prepended to the file name
   * @return the name of the file created
   */
  public String getX3D(X3DRepresentable[] x3d_objects, String imageOutputLocation, String filePrefix){
    String retVal = "unsuccessful";
    // Create a file and write the x3d to it
    try{
      File x3dOut = File.createTempFile(filePrefix + "_space",".x3d", new File(imageOutputLocation));
//      x3dOut.deleteOnExit();        // TODO Restore 'delete on exit' for map files
      FileOutputStream x3dOutput = new FileOutputStream(x3dOut.getAbsoluteFile());
      X3DAssist x3d = new X3DAssist();
      for(int i=0;i < x3d_objects.length; i++){    x3d.addX3DObject(x3d_objects[i]);    }
      x3dOutput.write(x3d.getX3D());
      x3dOutput.close();
      
      // Returns the name of the file created
      retVal = x3dOut.getName();
    }
    catch(Exception e){
      System.out.println(e.getMessage());
      e.printStackTrace();
      return "Could not create image";
    }
    return retVal; 
  }
    
  
  
}
