package flowag.basic;

import abcm.sim.AbstractModelComponent;

/**
 * Nutrients or chemicals carried by water or in soils.
 * Chemicals are specified by name and concentration; they are
 * always contained in other objects- waters or soils.<br/><br/>
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public abstract class AbstractChemical extends AbstractModelComponent implements Chemical {

  /** A name used to identify this chemical */        private String name;
  /** The concentration of this chemical, in ppm */   private double concentration;
  
    /**
     * Constructor that takes name and concentration as arguments
     * @param newName Name of the chemical
     * @param newConcentration Concentration of the chemical
     */
    public AbstractChemical(String newName, double newConcentration){
      name = newName;
      concentration = newConcentration;
    }
    
    /**
     * Gets the name of this chemical.
     * @return the name of this instance of a chemical
     */
    public String getName(){               return name;             }
    
    /**
     * Gets the current concentration of this chemical.
     * @return the concentration of this chemical 
     */
    public double getConcentration(){      return concentration;    }
    
    /**
     * Sets the concentration for this chemical.
     * @param newConcentration The new concentration for this chemical
     */
    public void setConcentration(double newConcentration){      concentration = newConcentration;      }
    
    /**
     * Allows adding one instance of a chemical object to another. The two objects
     * must have the same name. 
     * @param newChemical the chemical object to be added to this one
     * @return true if successful, false otherwise
     */
    public boolean addNew(Chemical newChemical){
      if(newChemical.getName().compareTo(name)!=0) return false;
      concentration += newChemical.getConcentration();
      return true;      
    }
    
}
