package flowag.basic;

import abcm.cloning.ABCMDeepCloneable;

/**
 * A basic definition of what any implementation of a Chemical must be able to do.
 *  
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public interface Chemical extends ABCMDeepCloneable{

  /**
   * Gets the name of this chemical.
   * @return the name of this instance of a chemical
   */
  public abstract String getName();

  /**
   * Gets the current concentration of this chemical.
   * @return the concentration of this chemical 
   */
  public abstract double getConcentration();

  /**
   * Sets the concentration for this chemical.
   * @param newConcentration The new concentration for this chemical
   */
  public abstract void setConcentration(double newConcentration);

  /**
   * Allows adding one instance of a chemical object to another. The two objects
   * must have the same name. 
   * @param newChemical the chemical object to be added to this one
   * @return true if successful, false otherwise
   */
  public abstract boolean addNew(Chemical newChemical);
  
}