package flowag.channels;

import java.util.Collection;
import java.util.Iterator;


/**
 * A Test class that implements a Flow Calculator 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class TESTFlowCalculator extends AbstractFlowCalculator {

  /** The collection of canal segments through which flow occurs  */
  Collection canalSegs;
  
  /**
   * Constructor
   * @param initialCanalSegment inflow segment
   * @param canalSegments collection of all segments
   */
  public TESTFlowCalculator(CanalSegment initialCanalSegment, Collection canalSegments){
    super(initialCanalSegment);
    canalSegs= canalSegments; 
  }
  
  /**
   * @see flowag.channels.AbstractFlowCalculator#performCalculations(double)
   */
  protected boolean performCalculations(double duration) {
    return true;
  }

  /**
   * @see flowag.channels.AbstractFlowCalculator#setResults()
   */
  protected void setResults() {
    results = new FlowCalculator.FlowZAndQAdjustments();
    
    Iterator I = canalSegs.iterator();
    while(I.hasNext()){
      CanalSegment cs = (CanalSegment)I.next();
      if(cs.equals(initialCanalSegment)) results.setInitialUpstreamValues(cs, 1, 0);
      results.addDownstreamValues(cs, .1, 0);
    }
  }

  /**
   * Does nothing- test class only
   * @param duration
   */
  protected void setResults(double duration) {
    
  }

}
