package flowag.db;

import flowag.basic.Water;
import abcm.db.ABCMdbDataInitializer;
import abcm.db.AbstractABCMdbPackageDataInitializer;

/**
 * FlowAgDBInitializer populates an ABCM database with the code model
 * and parameter information needed by the FlowAg routines. 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class FlowAgDBInitializer extends AbstractABCMdbPackageDataInitializer{
  
  /**
   * Flag to indicate whether the 'initialize' method of this class has been called.
   */
  private static boolean hasBeenInitialized = false;

  private static final String noCanalWearCode =  
    "case NO_WEAR:" + "\n" +
    "  default: {" + "\n" +
    "    // No action" + "\n" +
    "    break;" + "\n" +
    "  }";
  
  private static final String standardCanalWearCode = 
    "case STANDARD_WEAR:{" + "\n" + 
    "    // Roughness gets adjusted" + "\n" + 
    "    setRoughness(roughness + getParameterSet().getParameter(\"N_INC_PER_SECOND_PER_Q\") * time * getAverageDischarge());" + "\n" +
    "" + "\n" +    
    "    // Canal Fills in slightly or erodes out" + "\n" +
    "    double flowAverageVelocity = getUpstreamFlowVelocity() + getDownstreamFlowVelocity()/2d;" + "\n" + 
    "    if(flowAverageVelocity > getParameterSet().getParameter(\"EROSION_THRESHOLD\")){" + "\n" +
    "      excavate(getParameterSet().getParameter(\"CANAL_ERODE_PER_SECOND_PER_Q\") * time * getAverageDischarge());" + "\n" +
    "    }" + "\n" +
    "    if(flowAverageVelocity < getParameterSet().getParameter(\"INFILL_THRESHOLD\")){" + "\n" +
    "      infill(getParameterSet().getParameter(\"CANAL_FILL_PER_SECOND_PER_Q\") * time * getAverageDischarge());" + "\n" +
    "    }" + "\n" +
    "" + "\n" +            
    "    break;" + "\n" +
    "  }"; 
  
  private static final String noFlowCode = 
    "public boolean flow(double durationInDays){" + "\n" +
    "  switch(codeModelOption(\"FLOW_PROCEDURE\")){" + "\n" +
    "    case NO_FLOW:{" + "\n" +
    "      " + "\n" +
    "      }" + "\n" +
    "      break;"; 
  
  private static final String hecRasFlowProcedureCode = 
    "public boolean flow(double durationInDays){" + "\n" +
    "  switch(codeModelOption(\"FLOW_PROCEDURE\")){" + "\n" +
    "    /* HEC-RAS Flow - Implicit Finite Difference Approximation */" + "\n" +
    "    case HEC_RAS_IMP_FINITE_DIFF:{" + "\n" +
    "      System.out.println(\"Running HEC-RAS in Flow System\");" + "\n" +
    "      double totalTimeCalculated = 0;" + "\n" +
    "      while(totalTimeCalculated < durationInSeconds){" + "\n" +
    "        double dayFrac = totalTimeCalculated / (24d * 60d * 60d);" + "\n" +
    "  //          System.out.println(\"\nFLOWING AT TIME \" + totalTimeCalculated + \"/\" + dayFrac + \" current flow is \" + ((CanalSegment)this.canals.get(\"ProbeSeg1\")).getInflowGate().getHeadGate().getFlowAt(dayFrac));" + "\n" +
    "        " + "\n" +
    "        Vector flowSystems = getHECFlowSystems(dayFrac);" + "\n" +
    "  " + "\n" +
    "        // For each flow system, calculate the maximum time of flow possible, then find the lowest of these among all flow systems" + "\n" +
    "        double t = Math.min(MAXIMUM_TIMESTEP, (durationInSeconds - totalTimeCalculated));" + "\n" +
    "        for(int i = 0; i < flowSystems.size(); i++)" + "\n" +
    "          t = Math.min(t, ((HEC_RAS_FlowCalculator)flowSystems.get(i)).solveForMaxTimestep(t));" + "\n" +          
    "        " + "\n" +
    "        // Flow all flow systems for the lowest maximum time possible (ASSUMES STABILITY AT MIN!!!)" + "\n" +
    "        boolean success = true;" + "\n" +
    "        for(int i = 0; i < flowSystems.size(); i++)" + "\n" + 
    "            success = success & ((HEC_RAS_FlowCalculator)flowSystems.get(i)).calculate(t); // Will do all but will show success of all" + "\n" +
    "        if(!success) System.err.println(\"AT LEAST ONE FLOW SYSTEM FAILED FOR TIME \" + t);" + "\n" +
    "        " + "\n" +
    "        // Now we play a trick: we set the canal segments` discharge and flow values to" + "\n" + 
    "        // their current values plus _half_ of the delta values. Then we execute the" + "\n" +
    "        // move water and wear routines. Then we will add the second half of the delta values" + "\n" +
    "        // back in. The effect of this is that the wear and flow routines will use the" + "\n" +
    "        // median value across the change, effectively using average flow values to" + "\n" +
    "        // calculate the wear during the interval of time t." + "\n" +
    "        dayFrac = (totalTimeCalculated + t/2d) / (24d * 60d * 60d);" + "\n" +
    "        for(int i = 0; i < flowSystems.size(); i++) {" + "\n" +
    "          ((FlowCalculator)flowSystems.get(i)).getResults().applyFraction(canals.values(), .5d, dayFrac);" + "\n" +
    "        }" + "\n" +
    "        " + "\n" +
    "        // Move the water- this affects chemical composition and puts water on fields" + "\n" +
    "        moveWater(t);" + "\n" +
    "        " + "\n" +
    "        // This allows degradation of canal structure" + "\n" +
    "        degradeCanals(t);" + "\n" +
    "        " + "\n" +
    "        // Increment the overall clock" + "\n" +
    "        totalTimeCalculated += t;" + "\n" +
    "        dayFrac = totalTimeCalculated / (24d * 60d * 60d);" + "\n" +
    "        " + "\n" +
    "        // Now increase the discharges to the final setting, at the end of time t" + "\n" +
    "        for(int i = 0; i < flowSystems.size(); i++) " + "\n" +
    "            ((FlowCalculator)flowSystems.get(i)).getResults().applyFraction(canals.values(), .5d, dayFrac);" + "\n" +
    "        }" + "\n" +
    "      }" + "\n" +
    "        // Break out of case switch!" + "\n" +
    "      break;" + "\n";
  
  private static final String lpgFlowProcedureCode = 
    "public boolean flow(double durationInDays){" + "\n" +
    "  switch(codeModelOption(\"FLOW_PROCEDURE\")){" + "\n" +
    "    case LPG:{" + "\n" +
    "      double totalTimeCalculated = 0;" + "\n" +
    "      while(totalTimeCalculated < durationInSeconds){" + "\n" +
    "        double dayFrac = totalTimeCalculated / (24d * 60d * 60d);" + "\n" +
    "  //          System.out.println(\"\nFLOWING AT TIME \" + totalTimeCalculated + \"/\" + dayFrac);" + "\n" +
    "        " + "\n" +
    "        Vector flowSystems = getLPGFlowSystems(dayFrac);" + "\n" +
    "  " + "\n" +
    "        // For each flow system, calculate the maximum time of flow possible, then find the lowest of these among all flow systems" + "\n" +
    "        double t = Math.min(MAXIMUM_TIMESTEP, (durationInSeconds - totalTimeCalculated));" + "\n" +
    "        " + "\n" +
    "        // Flow all flow systems for the lowest maximum time possible (ASSUMES STABILITY AT MIN!!!)" + "\n" +
    "        boolean success = true;" + "\n" +
    "        for(int I = 0; I < flowSystems.size(); i++)" + "\n" + 
    "            success = success & ((FlowCalculator)flowSystems.get(i)).calculate(t); // Will do all but will show success of all" + "\n" +
    "        if(!success) System.err.println(\"AT LEAST ONE FLOW SYSTEM FAILED FOR TIME \" + t);" + "\n" +
    "        " + "\n" +
    "        // Now we play a trick: we set the canal segments` discharge and flow values to" + "\n" + 
    "        // their current values plus _half_ of the delta values. Then we execute the" + "\n" +
    "        // move water and wear routines. Then we will add the second half of the delta values" + "\n" +
    "        // back in. The effect of this is that the wear and flow routines will use the" + "\n" +
    "        // median value across the change, effectively using average flow values to" + "\n" +
    "        // calculate the wear during the interval of time t." + "\n" +
    "        dayFrac = (totalTimeCalculated + t/2d) / (24d * 60d * 60d);" + "\n" +
    "        for(int I = 0; I < flowSystems.size(); i++) {" + "\n" +
    "          ((FlowCalculator)flowSystems.get(i)).getResults().applyFraction(canals.values(), .5d, dayFrac);" + "\n" +
    "        }" + "\n" +
    "  //          System.out.println(\"Moving water\");" + "\n" +
    "        // Move the water- this affects chemical composition and puts water on fields" + "\n" +
    "        moveWater(t);" + "\n" +
    "  " + "\n" +          
    "        // This allows degradation of canal structure" + "\n" +
    "        degradeCanals(t);" + "\n" +
    "  " + "\n" +          
    "        // Increment the overall clock" + "\n" +
    "        totalTimeCalculated += t;" + "\n" +
    "        dayFrac = totalTimeCalculated / (24d * 60d * 60d);" + "\n" +
    "  " + "\n" +          
    "        // Now increase the discharges to the final setting, at the end of time t" + "\n" +
    "        for(int I = 0; I < flowSystems.size(); i++)" + "\n" + 
    "            ((FlowCalculator)flowSystems.get(i)).getResults().applyFraction(canals.values(), .5d, dayFrac);" + "\n" +        
    "        }" + "\n" +
    "      }" + "\n" +
    "      break;";
  
  private static final String noChannelEvapCode = 
    "public Water getOutflowWaterSample(){" + "\n" + 
    "  Water W = new Water(1, getInflowWaterSample());" + "\n" +
    "  switch(codeModelOption(\"EVAPORATION_ALGORITHM\")){" + "\n" +
    "  case NO_CHANNEL_EVAPORATION:{" + "\n" +
    "    return W;" + "\n" +
    "  }" + "\n";
  
  private static final String basicChannelEvapCode =
    "public Water getOutflowWaterSample(){" + "\n" +
    "Water W = new Water(1, getInflowWaterSample());" + "\n" +
    "switch(codeModelOption(\"EVAPORATION_ALGORITHM\")){" + "\n" +
    "  case BASIC_CHANNEL_EVAPORATION:{" + "\n" +
    "    double mmLossPerSecond = getParameterSet().getParameter(\"EVAPORATION_MM_PER_DAY\") / (24d * 60d * 60d);" + "\n" +
    "    double v = super.averageFlowVelocity();" + "\n" +
    "     double l = super.getLength();" + "\n" +
    "    double flowTime = (v != 0 ? (l / v) : 0);" + "\n" +
    "    double usd = getUpstreamWaterDepth();" + "\n" +
    "    double dsd = getDownstreamWaterDepth();" + "\n" +
    "    double afterFlowUpstreamDepth   = (v > 0 ? usd : usd - (flowTime * mmLossPerSecond)); // Note flow is directional!" + "\n" +
    "    double afterFlowDownstreamDepth = (v < 0 ? dsd : dsd - (flowTime * mmLossPerSecond));" + "\n" +
    "    " + "\n" +
    "     double volumeBeforeFlow = getTotalWater(usd, dsd);" + "\n" +
    "     double volumeAfterFlow  = getTotalWater(afterFlowUpstreamDepth, afterFlowDownstreamDepth);" + "\n" +
    "     W.evaporate((volumeBeforeFlow - volumeAfterFlow)/volumeBeforeFlow);" + "\n" +
    "     return W;" + "\n" +
    "  }" + "\n" +
    "}";
  
  private static final String noPlantGrowthCode = 
    "public void step(){" + "\n" +
    "  //  System.out.println(\"CROP STEP; \" + codeModelOption(\"PLANT_PRODUCTION\"));" + "\n" +
    "  switch(codeModelOption(\"PLANT_PRODUCTION\")){" + "\n" +
    "  case NO_PLANT_GROWTH: // This will be the default case" + "\n" +
    "  default:" + "\n" +
    "    cropAge++;" + "\n" +
    "    if(cropAge >= lifeSpan) alive = false;" + "\n" +
    "    break;" + "\n" +
    "}"; 
  
  private static final String plantGrowthModel1Code = 
    "public void step(){" + "\n" +
    "  //    System.out.println(\"CROP STEP; \" + codeModelOption(\"PLANT_PRODUCTION\"));" + "\n" +
    "  switch(codeModelOption(\"PLANT_PRODUCTION\")){" + "\n" +
    "    case PLANT_GROWTH_MODEL_1:" + "\n" +
    "      System.out.println(\"\n\nRUNNING PLANT GROWTH MODEL 1 FOR CROP \" + plant.getName() + \" current age = \" + cropAge + \" lifespan = \"  + lifeSpan);" + "\n" +
    "      cropAge++;                                                               // Increase the crop`s age" + "\n" +
    "      if(cropAge < lifeSpan){                                                  // Once lifespan reached, plant processes end" + "\n" +
    "        timeInCurrentStage++;                                                    // And the time in the current stage" + "\n" +
    "        // Determine if it is time to move to the next stage" + "\n" +
    "        if(timeInCurrentStage > durationOfCurrentStage){" + "\n" +
    "          currentStage++;" + "\n" +
    "          timeInCurrentStage               = 0;" + "\n" +
    "          droughtSensitivityOfCurrentStage = plant.getDroughtSensitivityOfStage(currentStage);" + "\n" +
    "          durationOfCurrentStage           = plant.getDurationOfStage(currentStage);" + "\n" +
    "          cropFactorOfCurrentStage         = plant.getCropFactorOfStage(currentStage);" + "\n" +
    "        }" + "\n" +
    "        " + "\n" +
    "        double waterNeed         = field.getETo() * cropFactorOfCurrentStage * density;" + "\n" +
    "        double minimumWaterNeed  = waterNeed * droughtSensitivityOfCurrentStage;" + "\n" +
    "        double waterAvailable    = field.getWaterAvailableMM();" + "\n" +
    "        double waterToRemove     = Math.min(waterNeed, waterAvailable);" + "\n" +
    "        " + "\n" +
    "        " + "\n" +
    "        if(waterAvailable < minimumWaterNeed){" + "\n" +
    "          System.out.println(\"CROP IN FIELD \" + field.name + \"                     HAS WATER SHORTAGE: \" + waterAvailable + \" vs. \" + minimumWaterNeed);" + "\n" +
    "          consecutiveDaysWaterNeedNotMet++;" + "\n" +
    "          maxConsecutiveDaysWaterNeedNotMet = Math.max(consecutiveDaysWaterNeedNotMet, maxConsecutiveDaysWaterNeedNotMet);" + "\n" +
    "          // Die?" + "\n" +
    "        }" + "\n" +
    "        else{" + "\n" +
    "  //            System.out.println(\"CROP IN FIELD                     \" + field.name + \" HAS ENOUGH WATER: \" + waterAvailable + \" vs. \" + minimumWaterNeed);" + "\n" +
    "          consecutiveDaysWaterNeedNotMet = 0;          " + "\n" +
    "        }" + "\n" +
    "        totalActualWater       += waterToRemove;" + "\n" +
    "        totalWaterNeed         += waterNeed;" + "\n" +
    "        totalMinimumWater      += minimumWaterNeed;" + "\n" +
    "        Water waterRemoved = field.removeWaterMM(waterToRemove);" + "\n" +
    "  //          System.out.println(\"REMOVING \" + waterToRemove);" + "\n" +
    "        // Now adjust the chemical need adjustment value based on soil" + "\n" +
    "        // chemistry" + "\n" +
    "        Enumeration E = plant.chemicalEffects.keys();" + "\n" +
    "        while(E.hasMoreElements()){" + "\n" +
    "          Plant.PlantChemicalEffect effect = (Plant.PlantChemicalEffect)plant.chemicalEffects.get((String)E.nextElement());" + "\n" +
    "          String chemical = effect.chemical;" + "\n" +
    "          double need = effect.need;" + "\n" +
    "          double present = (double)this.field.getChemicalConcentration(chemical);" + "\n" +
    "          System.out.print(\"Chemical is: \" + chemical + \" need is \" + need + \" present is \" + present);" + "\n" +
    "          if(((need > 0) && (present < need)) || ((need < 0) && (present > (Math.abs(need))))){" + "\n" +
    "            chemicalNeedAdjustment -= (.5d) / lifeSpan;" + "\n" +
    "            System.out.println(\" chemical need adjustment is \" + chemicalNeedAdjustment);" + "\n" +
    "            break; // Once it is short or over on any of them, the adjustment is made- don`t do any more!" + "\n" +
    "          }" + "\n" +
    "          else{System.out.println(\" need is met.\");}" + "\n" +
    "  " + "\n" +                        
    "        }" + "\n" +
    "        processChemicals();" + "\n" +
    "      }" + "\n" +
    "      else{" + "\n" +
    "        if(cropAge > lifeSpan * 1.25){" + "\n" +
    "          remove = true;  // Once crop is past harvestable, automatically remove self" + "\n" +
    "        }" + "\n" +
    "      }" + "\n" +
    "      break;" + "\n" +
    "}";
  
  private static final String noEvaporationCode = 
    "public void evaporate(){" + "\n" +
    "  if(getWaterAvailableMM() == 0) return;" + "\n" +
    "  switch(codeModelOption(\"SOIL_EVAPORATION\")){" + "\n" +
    "    case NO_EVAPORATION:" + "\n" +
    "    default:" + "\n" +
    "      break;" + "\n" +
    "  }" + "\n" +
    "}";
  
  private static final String basicEvaporationCode = 
    "public void evaporate(){" + "\n" +
    "  if(getWaterAvailableMM() == 0) return;" + "\n" +
    "  switch(codeModelOption(\"SOIL_EVAPORATION\")){" + "\n" +
    "    case BASIC_EVAPORATION:{" + "\n" +
    "      double mmToEvaporate = Math.min(getWaterAvailableMM(), paramVal(\"SOIL_EVAPORATE_FLAT_RATE\"));" + "\n" +
    "      Water w = extractWaterMM(mmToEvaporate);" + "\n" +
    "      List<Chemical> l = new ArrayList<Chemical>();" + "\n" +
    "      " + "\n" +
    "      if(w.getChemicals().size() > 0){" + "\n" +
    "        Enumeration E = w.getChemicals().elements();" + "\n" +
    "        while(E.hasMoreElements()){" + "\n" +
    "          Chemical C = (Chemical)E.nextElement();" + "\n" +
    "          C.setConcentration(C.getConcentration() * (mmToEvaporate / soilDepth_mm) * (paramVal(\"SOIL_CHEMICAL_EVAP_CONV_RATIO\")));" + "\n" +
    "          l.add(C);" + "\n" +
    "         }" + "\n" +
    "         addChemicals(l);" + "\n" +
    "      }" + "\n" +
    "      break;" + "\n" +
    "    }" + "\n" +
  "}";

  private static final String standardHeadgateEfficiencyCode =
    "public double getEfficiency(double flowLevelInMAFPerDay){" + "\n" +
    "  if(closed) return 0;" + "\n" +
    "    case STANDARD_HEADGATE_EFFICIENCY:" + "\n" +
    "    default:" + "\n" +
    "        return getEfficiency();";
  
  private static final String intervalHeadgateEfficiencyCode = 
    "public double getEfficiency(double flowLevelInMAFPerDay){" + "\n" +
    "  if(closed) return 0;" + "\n" +
    "  switch(codeModelOption(\"HEADGATE_EFFICIENCY\")){" + "\n" +
    "    case INTERVAL_HEADGATE_EFFICIENCY:" + "\n" +
    "      if(efficiencyRanges == null || efficiencyRanges.length == 0)  return getEfficiency();" + "\n" +
    "      for(int i = 0; i < efficiencyRanges.length; i++){" + "\n" +
    "        if(flowLevelInMAFPerDay < efficiencyRanges[i].upperBound){" + "\n" +
    "          return efficiencyRanges[i].efficiency; " + "\n" +
    "        }" + "\n" +
    "      }" + "\n" +
    "     return efficiencyRanges[efficiencyRanges.length - 1].efficiency; // If it is above the last range, return the last efficiency";
  
  private static final String interpolatedHeadgateEfficiencyCode = 
    "public double getEfficiency(double flowLevelInMAFPerDay){" + "\n" +
    "  if(closed) return 0;" + "\n" +
    "    case INTERPOLATED_HEADGATE_EFFICIENCY:" + "\n" +
    "      if(efficiencyRanges == null || efficiencyRanges.length == 0) return getEfficiency(); // This allows gates of both kinds" + "\n" +
    "      double lastEff = 0;" + "\n" +
    "      double lastBound = 0;" + "\n" +
    "      for(int I = 0; I < efficiencyRanges.length; i++){" + "\n" +
    "        if(flowLevelInMAFPerDay < efficiencyRanges[i].upperBound){" + "\n" +
    "          return lastEff + " + "\n" +
    "             (flowLevelInMAFPerDay - lastBound) *" + "\n" + 
    "                 (efficiencyRanges[i].efficiency - lastEff) / (efficiencyRanges[i].upperBound - lastBound);" + "\n" +
    "        }" + "\n" +
    "        lastEff = efficiencyRanges[i].efficiency;" + "\n" +
    "        lastBound = efficiencyRanges[i].upperBound;" + "\n" +
    "      }" + "\n" +
    "      return efficiencyRanges[efficiencyRanges.length - 1].efficiency; // If it is above the range, return the last value" + "\n" + 
    "}";
    
  private static final String standardSeepCode = 
    "public void seep(){" + "\n" +
    "  switch(codeModelOption(\"SEEP_PROCEDURE\")){" + "\n" +
    "    case NO_SEEPAGE:" + "\n" +
    "    default:{ // NO SEEP" + "\n" +
    "  //        System.out.println(\"No seepage occurring\");" + "\n" +
    "    }" + "\n" +
    "  }" + "\n" +
  "  }";
  
  private static final String basicSeepCode = 
    "public void seep(){" + "\n" +
    "  switch(codeModelOption(\"SEEP_PROCEDURE\")){" + "\n" +
    "    case BASIC_SEEP_PROCEDURE:{" + "\n" +
    "      if(getWaterAvailableMM() == 0) return; // If no water, done!" + "\n" +
    "      System.out.println(\"SEEPING\");" + "\n" +
    "      addChemicalsFromSeep(                                                 // Add the chemicals from the seeping water" + "\n" +
    "          extractWaterMM(                                                     // Remove the seeping water from the water object" + "\n" +
    "              Math.min(getWaterAvailableMM(), paramVal(\"SOIL_SEEP_FLAT_RATE\")))); // How much water" + "\n" +
    "      break;" + "\n" +
    "    }" + "\n" +
    "}";

  
  private static final String LPG_depth_Manning = 
    "protected void setResults(){" + "\n" +
    "  results = new FlowCalculator.FlowZAndQAdjustments();" + "\n" +
    "  " + "\n" +
    "  if(initialSegmentUpstreamQ == 0)  AbstractFlowCalculator.setToEmpty(this.initialCanalSegment, results);" + "\n" +
    "  " + "\n" +
    "  " + "\n" +
    "  for(CanalSegData csd: allSegs){" + "\n" +
    "    if(csd.upstreamSegmentData == null){ // It`s the initial segment" + "\n" +
    "      double newDepth = 0;" + "\n" +
    "      switch(codeModelOption(LPG_DEPTH_CALCULATION)){" + "\n" +
    "        case MANNING_REVERSE_EST:" + "\n" +
    "        default:" + "\n" +
    "          newDepth = csd.canalSeg.getUpstreamDepthPerManningDischarge(initialSegmentUpstreamQ); // THIS IS SLOW!" + "\n" +
    "          break;" + "\n" +
    "      }" + "\n" +
    "      results.setInitialUpstreamValues(csd.canalSeg," + "\n" + 
    "                                       newDepth - csd.canalSeg.getUpstreamWaterDepth()," + "\n" + 
    "                                       initialSegmentUpstreamQ - csd.canalSeg.getUpstreamDischarge());" + "\n" +
    "    }" + "\n" +
    "    double newDepth = 0;" + "\n" +  
    "    switch(codeModelOption(LPG_DEPTH_CALCULATION)){" + "\n" +
    "      case MANNING_REVERSE_EST:" + "\n" +
    "      default:" + "\n" +
    "        newDepth = csd.canalSeg.getDownstreamDepthPerManningDischarge(csd.newQ); // THIS IS SLOW!" + "\n" +
    "        break;" + "\n" +
    "    }" + "\n" +
    "    results.addDownstreamValues(csd.canalSeg," + "\n" + 
    "                              newDepth - csd.canalSeg.getDownstreamWaterDepth()," + "\n" + 
    "                              csd.newQ - csd.canalSeg.getDownstreamDischarge() );" + "\n" +
    "  }" + "\n" +
    "}";
  
  private static final String LPG_depth_Fraction = 
    "protected void setResults(){" + "\n" +
    "  results = new FlowCalculator.FlowZAndQAdjustments();" + "\n" +
    "  " + "\n" +
    "  if(initialSegmentUpstreamQ == 0)  AbstractFlowCalculator.setToEmpty(this.initialCanalSegment, results);" + "\n" +
    "  " + "\n" +
    "  " + "\n" +
    "  for(CanalSegData csd: allSegs){" + "\n" +
    "    if(csd.upstreamSegmentData == null){ // It is the initial segment" + "\n" +
    "      double newDepth = 0;" + "\n" +
    "      switch(codeModelOption(LPG_DEPTH_CALCULATION)){" + "\n" +
    "        case FRACTION_OF_CHANNEL:" + "\n" +
    "          newDepth = (csd.newQ > 0 ? csd.canalSeg.getUpstreamChannelHeight() * .667 : 0);" + "\n" +
    "          break;" + "\n" +
    "      }" + "\n" +
    "      results.setInitialUpstreamValues(csd.canalSeg," + "\n" + 
    "                                       newDepth - csd.canalSeg.getUpstreamWaterDepth()," + "\n" + 
    "                                       initialSegmentUpstreamQ - csd.canalSeg.getUpstreamDischarge());" + "\n" +
    "    }" + "\n" +
    "    double newDepth = 0;" + "\n" +  
    "    switch(codeModelOption(LPG_DEPTH_CALCULATION)){" + "\n" +
    "      case FRACTION_OF_CHANNEL:" + "\n" +
    "        newDepth = (csd.newQ > 0 ? csd.canalSeg.getDownstreamChannelHeight() * .667 : 0);" + "\n" +
    "        break;" + "\n" +
    "    }" + "\n" +
    "    results.addDownstreamValues(csd.canalSeg," + "\n" + 
    "                              newDepth - csd.canalSeg.getDownstreamWaterDepth()," + "\n" + 
    "                              csd.newQ - csd.canalSeg.getDownstreamDischarge() );" + "\n" +
    "  }" + "\n" +
    "}";
  
  
  /**
   * Basic constructor
   * @param dataInitializer
   */
  public FlowAgDBInitializer(ABCMdbDataInitializer dataInitializer){
    super(dataInitializer);
  }
  
  /**
   * 
   * @see abcm.db.AbstractABCMdbPackageDataInitializer#hasBeenInitialized()
   */
  protected boolean hasBeenInitialized(){ return hasBeenInitialized; }
  
  /**
   * 
   * @see abcm.db.AbstractABCMdbPackageDataInitializer#setHasBeenInitialized()
   */
  protected void setHasBeenInitialized(){ hasBeenInitialized = true; }
  
  /**
   * Add the FlowAg data to the specified ABCM database
   * @throws Exception General exception to be handled by calling routine
   */
  protected void initialize() throws Exception{
    System.out.println("Adding FlowAg data...");
    
    try {
      // Parameters
      long paramCat = addParameterCategory("FlowAg", "FlowAg package");
      addParameter("LABOR_TO_CLEAN_CANAL", "Amount of labor to clean a canal", 1, 1, 10, "unknown", paramCat);
      addParameter("LABOR_TO_MOVE_1M3_DIRT", "Amount of labor to move 1 cubic meter dirt", 1, 1, 10, "unknown", paramCat);
      addParameter("EVAPORATION_MM_PER_DAY", "Evaporation rate from open canals", 1, 1, 10, "mm/day", paramCat);
      addParameter("DEBUG", "Debugging level", 0, 0, 5, "none", paramCat);
      addParameter("ROUNDING_DECIMAL_PLACES", "Number of decimal places to round numbers", 0, 0, 10, "none", paramCat);
      addParameter("N_INC_PER_SECOND_PER_Q", "Increase in roughness, per second, for given flow rate", 0, 0, 1, "none", paramCat);
      addParameter("CANAL_FILL_PER_SECOND_PER_Q", "Rate at which sediment is deposited, per flow rate", 0, 0, 1, "none", paramCat);
      addParameter("SOIL_EVAPORATE_FLAT_RATE", "Base rate of water evaporation from soil", 0, 0, 10, "mm", paramCat);
      addParameter("SOIL_SEEP_FLAT_RATE", "Rate at which water seeps from soil", 0, 0, 10, "mm", paramCat);
      addParameter("THETA", "Dimensionless scalar used in HEC-RAS calculations", 1, .01, 1, "none", paramCat);
      addParameter("SCALE_STREAMFLOW_SET_VALUE", "Dimensionless scalar multiple for streamflow value", 1, 0.000001, 100000, "none", paramCat);
      addParameter("DEFAULT_SOIL_DEPTH", "Default depth of soil in fields",1, .01, 100, "meters", paramCat );
      addParameter("SOIL_CHEMICAL_CONCENTRATION_LIMIT", "Limit of soil chemical concentration", 300000, 100000, 1000000, "ppm", paramCat);
      addParameter("SOIL_CHEMICAL_EVAP_CONV_RATIO", "Convert concentration in evaporated water to soil", .01, .00001, 1, "none", paramCat);
      addParameter("SOIL_CHEMICAL_SEEP_CLEAN_LIMIT", "Limit below which water can flush soils", 100, 0, 1000000, "ppm", paramCat);
      addParameter("SOIL_CHEMICAL_SEEP_CONV_RATIO", "Convert concentration in seeped water to soil", .01, .00001, 1, "none", paramCat);
      
      // Code Model Domains and options
      String code = "";
      String description = "-description TBD-";
      long domainVal;
 
      // Canal Wear
      domainVal = addCodeModelDomain("Canal Wear", "Determines how canals erode", "CANAL_WEAR");
        addCodeModelDomainOptionVal(domainVal, 0, "NO_WEAR", noCanalWearCode, "No erosion occurs");
        addCodeModelDomainOptionVal(domainVal, 1, "STANDARD_WEAR", standardCanalWearCode, "Standard method for calculating canal wear");
        
        
      // Flow Procedure
      domainVal = addCodeModelDomain("Flow Procedure", "Determines how flow is calculated through a canal system", "FLOW_PROCEDURE");
        addCodeModelDomainOptionVal(domainVal, 0, "NO_FLOW", noFlowCode, "No flow occurs");
        addCodeModelDomainOptionVal(domainVal, 1, "HEC_RAS_IMP_FINITE_DIFF", hecRasFlowProcedureCode, "Uses the HEC-RAS implicite finite difference approximation code");
        addCodeModelDomainOptionVal(domainVal, 2, "LPG", lpgFlowProcedureCode, "Uses the `looks pretty good` approach to determining how water is distributed among fields.");
      
      // Flow Evaporation
      domainVal = addCodeModelDomain("Channel Evaporation", "Determines how water evaporates from flowing channels", "EVAPORATION_ALGORITHM");
        addCodeModelDomainOptionVal(domainVal, 0, "NO_CHANNEL_EVAPORATION", noChannelEvapCode, "No evaporation occurs from flowing channels");
        addCodeModelDomainOptionVal(domainVal, 1, "BASIC_CHANNEL_EVAPORATION", basicChannelEvapCode, "Basic channel evaporation algorithm");
      
      // Plant Productivity
      domainVal = addCodeModelDomain("Plant Production", "", "PLANT_PRODUCTION");
        addCodeModelDomainOptionVal(domainVal, 0, "NO_PLANT_GROWTH", noPlantGrowthCode, "No plant growth occurs");
        addCodeModelDomainOptionVal(domainVal, 1, "PLANT_GROWTH_MODEL_1", plantGrowthModel1Code, description);
      
      // ETo Method
      domainVal = addCodeModelDomain("Soil Seep", "", "SEEP_PROCEDURE");
        addCodeModelDomainOptionVal(domainVal, 0, "NO_SEEPAGE", standardSeepCode, "No seepage");
        addCodeModelDomainOptionVal(domainVal, 1, "BASIC_SEEP_PROCEDURE", basicSeepCode, "Seepage occurs at flat rate");
      
      
      // Soil Evaporation
      domainVal = addCodeModelDomain("Soil Evaporation", "", "SOIL_EVAPORATION");
        addCodeModelDomainOptionVal(domainVal, 0, "NO_EVAPORATION", noEvaporationCode, "No evaporation occurs");
        addCodeModelDomainOptionVal(domainVal, 1, "BASIC_EVAPORATION", basicEvaporationCode, description);
      
      
        
      // Calculating Headgate Efficiency
      domainVal = addCodeModelDomain("Headgate Efficiency", "Determines how sim calculates the efficiency of a headgate in removing flow from the river","HEADGATE_EFFICIENCY");
        addCodeModelDomainOptionVal(domainVal, 0, "STANDARD_HEADGATE_EFFICIENCY", standardHeadgateEfficiencyCode, "Uses a single scalar efficiency value");
        addCodeModelDomainOptionVal(domainVal, 1, "INTERVAL_HEADGATE_EFFICIENCY", intervalHeadgateEfficiencyCode, "Uses a user-specified set of ranges to specify headgate efficiency");
        addCodeModelDomainOptionVal(domainVal, 2, "INTERPOLATED_HEADGATE_EFFICIENCY", interpolatedHeadgateEfficiencyCode, "Uses a user-specified set of ranges to calculate efficiency; interpolates between end points of range");
              
      // Calculating depth in LPG algorithm
      domainVal = addCodeModelDomain("LPG Depth Calcs", "Determines how sim calculates depth after using LPG to calculate Q ", "LPG_DEPTH_CALCULATION");
        addCodeModelDomainOptionVal(domainVal, 0, "MANNING_REVERSE_EST", LPG_depth_Manning, "Uses reverse-estimation to calculate depth based on Manning equation");
        addCodeModelDomainOptionVal(domainVal, 1, "FRACTION_OF_CHANNEL", LPG_depth_Fraction, "Arbitrarily assigns depth as constant fraction of channel");
        
        
    } catch (RuntimeException e) {
      System.out.println("ERROR Adding FlowAg data!");
      throw(e); // Toss back up to higher-level handler which will roll back db
    }   
    
    System.out.println("Done adding FlowAg data.");
  }
  
  
  /**
   * Adds a code model domain option value. Automatically chooses values it assumes will be unique for keys.
   * @param domainVal integer that indicates the domain being used
   * @param index index for value within that domain
   * @param ID ID for the option value
   * @param code code to be inserted
   * @param desc description
   * @return the PK of the new entry in the database
   * @throws Exception generic exception
   */
  protected long addCodeModelDomainOptionVal(long domainVal, int index, String ID, String code, String desc) throws Exception{
     return addCodeModelDomainOptionValue(domainVal * 100 + index + 1, domainVal, index, ID, code, desc);
  }
}
