package wm.db;

import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMObjectCommentTable;
import abcm.db.components.ABCMdbField;
import abcm.db.managers.CommentManager;
import abcm.db.managers.abstractparents.AbstractDBManager;
import abcm.db.managers.abstractparents.AbstractObjectCommentManager;

/**
 * Class that manages comments in a WatMan-based ABCM database
 *  
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class WMCommentManager extends CommentManager{
  
  /**
   * Manager for plant-related comments
   * @author John T. Murphy<br/>
   *  Global Institute of Sustainability,<br/>
   *  Arizona State University<br/>
   *  For license and disclaimers, see the Overview.
   */
  private class PlantCommentManager extends AbstractObjectCommentManager{

    /**
     * Constructor
     * @param db DBGateway object to be used
     * @param containedBy encapsulating class
     */
    public PlantCommentManager(AbstractABCMDBGateway db, AbstractDBManager containedBy) {
      super(db, containedBy);
    }

    /** 
     * @see abcm.db.managers.abstractparents.AbstractObjectCommentManager#getBaseTable() 
     */
    public ABCMObjectCommentTable getBaseTable(){      return (ABCMObjectCommentTable)DB().getPlantCommentsTable();    }
    
    /** 
     * @see abcm.db.managers.abstractparents.AbstractObjectCommentManager#getCommentIDField() 
     */
    public ABCMdbField getCommentIDField(){            return DB().getPlantCommentsTable().getField_CommentFK();       }

    /** 
     * @see abcm.db.managers.abstractparents.AbstractObjectCommentManager#getObjectIDField() 
     */
    public ABCMdbField getObjectIDField(){             return DB().getPlantCommentsTable().getField_PlantFK();         }
        
  }
  
  /**
   * Manager for Plant Configuration comments
   * 
   * @author John T. Murphy<br/>
   * 
   *  Global Institute of Sustainability,<br/>
   *  Arizona State University<br/>
   *  For license and disclaimers, see the Overview.
   */
  private class PlantConfigurationCommentManager extends AbstractObjectCommentManager{

    /**
     * Constructor
     * @param db DBGateway object to be used
     * @param containedBy encapsulating class
     */
    public PlantConfigurationCommentManager(AbstractABCMDBGateway db, AbstractDBManager containedBy) {
      super(db, containedBy);
    }

    /** 
     * @see abcm.db.managers.abstractparents.AbstractObjectCommentManager#getBaseTable() 
     */
    public ABCMObjectCommentTable getBaseTable(){     return DB().getPlantConfigurationCommentsTable();                                    }

    /** 
     * @see abcm.db.managers.abstractparents.AbstractObjectCommentManager#getCommentIDField() 
     */
    public ABCMdbField getCommentIDField(){           return DB().getPlantConfigurationCommentsTable().getField_CommentFK();               }

    /** 
     * @see abcm.db.managers.abstractparents.AbstractObjectCommentManager#getObjectIDField() 
     */
    public ABCMdbField getObjectIDField(){            return DB().getPlantConfigurationCommentsTable().getField_PlantConfigurationFK();    }
    
  }
  
  /**
   * Constructor
   * @param DB WMDBGateway object to be used
   */
  public WMCommentManager(WMDBGateway DB){
    super(DB);
    addObjectCommentManager(WMDBGateway.PLANT,               new PlantCommentManager(DB, this));
    addObjectCommentManager(WMDBGateway.PLANT_CONFIGURATION, new PlantConfigurationCommentManager(DB, this));
  }
  
  /**
   * Gets the WMDBGateway object for this manager
   * @return a WMDBGateway object
   */
  public WMDBGateway DB(){ return ((WMDBGateway)DB); }
  
}
