package wm.db;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;

import wm.db.views.WMSimulationConfigurationPlantConfigurationsView;
import abcm.db.AbstractDBGateway;
import abcm.db.managers.stubs.AbstractSummaryManager;
import abcm.utilities.XMLAssist;

/**
 * WMSummaryManager provides database interface for WatMan-specific
 * summaries- i.e. summaries that include Plant Configurations.
 *  
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class WMSummaryManager extends AbstractSummaryManager {

  /**
   * Basic constructor
   * @param DB DatabaseGateway to be used by this manager
   */
  public WMSummaryManager(WMDBGateway DB){    
    super(DB);    
  }

  /**
   * Private method that permits easy access to the WMDBGateway-cast
   * of the superclass's DB variable.
   * @return WatMan DB Gateway to be used to get access to database
   */
  private WMDBGateway DB(){ return (WMDBGateway)DB; }
 
  /**
   * Gets the list of WatMan specific configurations as an XML List
   * @see abcm.db.managers.stubs.AbstractSummaryManager#getSpecificConfigurationsList(long)
   * @return XML description of the list of configurations
   */
  public XMLAssist getSpecificConfigurationsList(long simConfigID){
    System.out.println("GETTING SPECIFIC CONFIGURATIONS!");
    XMLAssist xml = new XMLAssist();
    try{
      // Get the plant configurations included in this configuration
//      String sql = "SELECT * FROM jtmurphy.SimulationConfigurationPlantConfigurations WHERE intSimulationConfigurationID = " + simConfigID + ";";
      WMSimulationConfigurationPlantConfigurationsView view = DB().getSimulationConfigurationPlantConfigurationsView();
      String sql = view.getSelectSQL(DB().getOwner(), AbstractDBGateway.where(view.getField_SimulationConfiguration_PK().getName(), simConfigID));
      Connection con = super.DB.getGuestConnection();
      
      Statement st = con.createStatement();
      ResultSet rs = st.executeQuery(sql);
      xml.open("plantConfigsInList");
        while(rs.next()){
          xml.open("plantConfigurationInList");
            xml.addLine("plantConfig", rs.getString("vcharPlantConfigName"));
            xml.addLine("plantConfigPlant", rs.getString("vcharPlantName"));
            xml.addLine("txtTruthValue", rs.getString("txtTruthValue"));
          xml.closeTag();
        }
      xml.closeTag(); // plantConfigs
      xml.closeTag(); // configurationInList
      rs.close();
      st.close();
      con.close();
    }
    catch(Exception e){
      xml = new XMLAssist();
      xml.addBlock("Configurations unavailable", XMLAssist.INDENT_BLOCK);
      addToErrorReport("An error occurred getting the specific configurations list: " + e.getMessage());
    }
    return xml;
  }
  
  /**
   * Gets the details of the WatMan specific configurations in the database.
   * @return XML description of the list of configurations.
   * @see abcm.db.managers.stubs.AbstractSummaryManager#getSpecificConfigurationsDetails(long)
   */
  public XMLAssist getSpecificConfigurationsDetails(long PK){
    XMLAssist xml = new XMLAssist();
    try{
      // Get the list of plant configurations
      String sql = DB().getSummaryPlantConfigurationsAndDetailsView().getSelectSQL(DB.getOwner(), 
          AbstractDBGateway.where(DB().getSummaryPlantConfigurationsAndDetailsView().getField_Summary_PK().getName(), PK));
      Connection con = super.DB.getGuestConnection();
      Statement st = con.createStatement();
      ResultSet rs = st.executeQuery(sql);
      
      PlantConfigurationManager pcm = new PlantConfigurationManager(DB());
      
      xml.open("plantConfigurationsListed");
        while(rs.next()){
          xml.addBlock(pcm.getInstanceXML(rs.getInt("intPlantConfigurationID"), WMConfigurationManager.EXCLUDE_INSTANCE_HEADER, WMConfigurationManager.INCLUDE_COMMENTS), XMLAssist.INDENT_BLOCK);
        }
      xml.closeTag();
      rs.close();
      st.close();
      con.close();
    }
    catch(Exception e){
      xml = new XMLAssist();
      xml.addBlock("Configuration details unavailable", XMLAssist.INDENT_BLOCK);
      addToErrorReport("An error occurred getting specific configuration details: " + e.getMessage());
    }
    return xml;
  }

}
