
package wm.db.views;

import wm.db.WMDBGateway;
import wm.db.tables.WMPlantCommentsTable;
import wm.db.tables.WMPlantsTable;
import abcm.db.AbstractABCMDBGateway;
import abcm.db.components.ABCMdbField;
import abcm.db.components.ABCMdbView;
import abcm.db.components.tables.ABCMCommentsTable;
import abcm.db.components.tables.ABCMReferenceCommentsTable;
import abcm.db.components.tables.ABCMReferencesTable;
import abcm.db.components.tables.ABCMResearchersTable;

public abstract class WMPlantsAndReferencesToThemView extends ABCMdbView {
  
  public WMPlantsAndReferencesToThemView(String viewName, AbstractABCMDBGateway db){ 
    super(viewName, db);
  }

  public abstract ABCMdbField getField_Plant_PK();
  public abstract ABCMdbField getField_Reference_PK();
  public abstract ABCMdbField getField_Reference_Title();
  public abstract ABCMdbField getField_Reference_Date();
  public abstract ABCMdbField getField_Reference_Pages();
  public abstract ABCMdbField getField_Reference_Comment();
  public abstract ABCMdbField getField_Researcher_Name();
  public abstract ABCMdbField getField_Comment_PK();
  
  public String getSQL(){
    String dbOwner = DB.getOwner();
    
    WMPlantsTable plantsTable = ((WMDBGateway)DB).getPlantsTable();
    
    ABCMReferencesTable referencesTable = DB.getReferencesTable();

    ABCMReferenceCommentsTable referenceCommentsTable = DB.getReferenceCommentsTable();
    
    ABCMCommentsTable commentsTable = DB.getCommentsTable();
    
    ABCMResearchersTable researchersTable = DB.getResearchersTable();
    
    WMPlantCommentsTable plantCommentsTable = ((WMDBGateway)DB).getPlantCommentsTable();
    
    return
      "SELECT " +plantsTable.getField_PK().getFullFieldName(dbOwner) + " AS " + getField_Plant_PK().getName() + ", " +"\n" + 
                   referencesTable.getField_PK().getFullFieldName(dbOwner) + " AS " + getField_Reference_PK().getName() + ", " +"\n" + 
                   referencesTable.getField_Title().getFullFieldName(dbOwner) + " AS " + getField_Reference_Title().getName() + ", " +"\n" + 
                   referencesTable.getField_ReferenceDate().getFullFieldName(dbOwner) + " AS " + getField_Reference_Date().getName() + ", " +"\n" + 
                   referenceCommentsTable.getField_Pages().getFullFieldName(dbOwner) + " AS " + getField_Reference_Pages().getName() + ", " +"\n" + 
                   referenceCommentsTable.getField_CommentFK().getFullFieldName(dbOwner) + " AS " + getField_Reference_Comment().getName() + ", " +"\n" + 
                   researchersTable.getField_ResearcherName().getFullFieldName(dbOwner) + " AS " + getField_Researcher_Name().getName() + ", " +"\n" + 
                   commentsTable.getField_PK().getFullFieldName(dbOwner) + " AS " + getField_Comment_PK().getName() + "\n" +  
      "FROM  " + researchersTable.getFullName(dbOwner) + " \n" +
        "INNER JOIN (" + plantsTable.getFullName(dbOwner) + " \n" +
            "INNER JOIN " +  plantCommentsTable.getFullName(dbOwner) + " ON " +"\n" + 
                    plantsTable.getField_PK().getFullFieldName(dbOwner) + " = " + plantCommentsTable.getField_PlantFK().getFullFieldName(dbOwner) + "\n" +
            "INNER JOIN " + commentsTable.getFullName(dbOwner) + " ON " +"\n" +
                    plantCommentsTable.getField_CommentFK().getFullFieldName(dbOwner) + " = " + commentsTable.getField_PK().getFullFieldName(dbOwner) + ") ON " +"\n" + 
                    researchersTable.getField_PK().getFullFieldName(dbOwner) + " = " + commentsTable.getField_ResearcherFK().getFullFieldName(dbOwner) + " \n" +
             "LEFT OUTER JOIN (" + referencesTable.getFullName(dbOwner) + " \n" +
             "INNER JOIN " + referenceCommentsTable.getFullName(dbOwner) + " ON " +"\n" + 
                    referencesTable.getField_PK().getFullFieldName(dbOwner) + " = " + referenceCommentsTable.getField_ReferenceFK().getFullFieldName(dbOwner) + ") ON " +"\n" + 
                    commentsTable.getField_PK().getFullFieldName(dbOwner) + " = " + referenceCommentsTable.getField_CommentFK().getFullFieldName(dbOwner) + " \n" +  
             "ORDER BY " + plantsTable.getField_PK().getFullFieldName(dbOwner) + ", \n" +
                           referencesTable.getField_ReferenceDate().getFullFieldName(dbOwner) + ",\n" +
                           referencesTable.getField_PK().getFullFieldName(dbOwner) + ",\n" +
                           referenceCommentsTable.getField_Pages().getFullFieldName(dbOwner);
  }


}
