package wm.model;

import wm.db.WMConfigurationManager;
import abcm.db.managers.stubs.AbstractConfigurationManager;
import abcm.model.AbstractABCModel;
import abcm.model.AbstractABCModelEnvironment;
import abcm.sim.SimulationRun;
import abcm.web.ABCMSession;

/**
 * Environment object in which the simulation will be run
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public abstract class WMModelEnvironment extends AbstractABCModelEnvironment{

  /** WatMan-specific Configuration Manager*/ protected WMConfigurationManager  CM;

  /* Constructors */
  /**
   * Constructor that requires an int that identifies the
   * mode in which this Environment is created (GUI mode,
   * stand-alone application model, etc.
   * @param environmentMode mode identifier
   */
  public WMModelEnvironment(int environmentMode){
    super(environmentMode);    
  }
  
  /**
   * Constructor that takes a mode identifier and an HWM Session
   * object
   * @param s Session object to be associated with this environment
   * @param environmentMode mode identifier
   */
  public WMModelEnvironment(ABCMSession s, int environmentMode){
    super(s, environmentMode);
  }

  /* Fundamentals */
  /**
   * 
   * @see abcm.model.AbstractABCModelEnvironment#getConfigurationManager()
   */
  public final AbstractConfigurationManager getConfigurationManager(){
    if(CM == null) CM = new WMConfigurationManager(getDBGateway());
    return CM;                    
  }
  
  /**
   * 
   * @see abcm.model.AbstractABCModelEnvironment#newModel()
   */
  public final AbstractABCModel newModel()                           { return new WMModel(this);     }
  
  /**
   * 
   * @see abcm.model.AbstractABCModelEnvironment#newModel(abcm.sim.SimulationRun)
   */
  public final AbstractABCModel newModel(SimulationRun sr)           { return new WMModel(this, sr); }
    
}