package wm.model.agents;

import java.util.List;
import java.util.Vector;

import wm.model.WMAgent;
import wm.model.WMModel;

/**
 * Manages a canal system by selecting the gates to distribution
 * canals that are either open or closed at any given time step.
 * As such this agent acts as a kind of central authority over the
 * individual agents along a canal system.
 * 
 *  @author John T. Murphy<br/>
 *  Global Institute of Sustainability,<br/>
 *  Arizona State University<br/>
 *  For license and disclaimers, see the Overview.
 */
public class WMCanalSystemManagerAgent extends WMAgent{
  
  /** Name of the canal system being managed */
  protected String canalSystemName;
  /** ID of this agent*/
  protected String ID;
  /** Collection of gates managed by this agent*/
  protected Vector<String> gates = new Vector<String>();
  /** Timekeeping index, allowing actions to be taken at intervals*/
  protected int index = 0;
  /** Timekeeping index, allowing actions to be taken at intervals*/
  protected int day = 0;
  
  /**
   * Constructor. Initializes collection of gates based on information from CanalSystem (via WMModel)
   * @param model model in which this agent exists
   * @param population population in which this agent exists
   * @param csName CanalSystem this agent will manage.
   */
  public WMCanalSystemManagerAgent(WMModel model, WMPopulation population, String csName) {
    super(model, population);
    canalSystemName = csName;
    List<String> distributionCanalInitialSegNames = (List<String>)model.getCSDistribCanInitSegNames(csName).object;
    gates = new Vector<String>(distributionCanalInitialSegNames);
  }
  
  
  /**
   * Manages canal system gates
   * @see wm.model.WMAgent#step()
   */
  public void step(){
    super.step();
    System.out.println("Running canal system manager step; gates = " + gates.size());
    if(gates.size() == 0) return;
    
    day++;
    if(day == 3){
      index++;
      if(index == gates.size() - 1) index = 0;
      for(int x = 0; x < gates.size() - 1; x++){
        if(x != index) out(myModel.processCommand("CloseInGate " + canalSystemName + " " + gates.get(x)).text);
      }
    }
    if(day == 4){
      day = 0;
      for(String gate: gates) out(myModel.processCommand("OpenInGate " + canalSystemName + " " + gate).text);  
    }
    
    // Obsolete algorithms
//    int c = (int)this.paramVal("NUMBER_DIST_CANALS_CLOSED");
//    for(int x = 0; x < c; x++){
//      int ind = index + x;
//      if(ind >= gates.size()) ind -= gates.size();
//      out(myModel.processCommand("OpenInGate " + canalSystemName + " " + gates.get(ind)).text);
//    }
//    // Update the index
//    index++;
//    if(index >= gates.size()) index = 0;
//    
//    for(int x = 0; x < c; x++){
//      int ind = index + x;
//      if(ind >= gates.size()) ind -= gates.size();
//      out(myModel.processCommand("CloseInGate " + canalSystemName + " " + gates.get(ind)).text);
//    }    
  }
  
}
